/*
 * Copyright (C) 2003  Vizi Gabor (vizi@csucs.net)
 *
 * Ez az adatallomany a reformat program forraskodjanak resze.
 *
 * A program hasznalata, masolasa, terjesztese maganszemely 
 * bevetelt nem eredmenyezo celjara az eredeti forras, 
 * a szerzoi jogi megjegyzes es a hasznalati feltetelek 
 * feltuntetesevel es valtozatlanul hagyasaval, szabadon lehetseges. 
 * A program minden garancia nelkul kerul kozreadasra, a program
 * mukodesebol fakado osszes kockazat a felhasznalot terheli. 
 */
#define COMMANDLINE_INTERNAL_ACCESS
#include "include.h"
#include "commandline.h"
#include "dataflow.h"
#include "conf.h"
#include "globals.h"
#include "output.h"
#include "cache.h"

static t_cline cline[]=
{
 {"--help", 0, do_help},
 {"-h", 0, do_help},
 {"--config=", 0, do_config},
 {"-c", 1, do_config},
 {"--set", 1, do_set},
 {"-s", 1, do_set},
 {"--out=", 0, do_output},
 {"-o", 1, do_output},
 {"--in=", 0, do_input},
 {"-i", 1, do_input},
 {"--version", 0, do_version},
 {"-v", 0, do_version},
 {"--quiet", 0, do_quiet},
 {"-q", 0, do_quiet},
 {"--pdirconf", 0, do_pdirconf},
 {"-p", 0, do_pdirconf},
#ifdef WIN32
 {"--nopause", 0, do_nopause},
#endif
 {"--cache", 2, do_cache},
 {"--cachedebug", 0, do_cachedebug},
 {"--psziernyo", 0, do_psziernyo},
 {NULL, 0, 0}
};

extern int read_commandline(int argc, char ** argv)
{
    int a;
    int aa;
    int i;
    char * parm;
    char * parm2;
    char * p;
    int ok;
    int ret;
    int ret2;
    char sep;
    char tmp;
    char * pdir;
    
    ret = 0;
    ret2 = 0;
    pdir = NULL;
    
#ifndef WIN32
    sep = '/';
#else
    sep = '\\';
#endif
    
    if (p = strrchr(argv[0], sep))
    {
	tmp = *p;
	*p = '\0';
	pdir = strdup(argv[0]);
	*p = tmp;
	set_pdir(pdir);
    }
    
    for (aa = 1, a = 1, parm = NULL, parm2 = NULL, ok = 0; a < argc; a++)
    {
	for (i = 0; cline[i].nev && cline[i].func && (i < sizeof(cline)/sizeof(t_cline)); i++, parm = NULL, parm2 = NULL, ok = 0)
	{
	    if (strncmp(argv[a], cline[i].nev, strlen(cline[i].nev)) == 0)
	    {
		if (cline[i].argc == 0)
		{
		    p = argv[a] + strlen(cline[i].nev) - 1;
		    if (*p == '=')
		    {
			parm = p + 1;
			argv[a] = NULL;
			aa++;
			ok = 1;
			break;
		    }
		    else if (strlen(argv[a]) == strlen(cline[i].nev))
		    {
			parm = NULL;
			argv[a] = NULL;
			aa++;
			ok = 1;
			break;
		    }
		}
		else if ((strlen(argv[a]) == strlen(cline[i].nev)) && (cline[i].argc == 1) && (argc - a >= 2))
		{
		    argv[a] = NULL;
		    a++;
		    aa++;
		    parm = argv[a];
		    argv[a] = NULL;
		    aa++;
		    ok = 1;
		    break;
		}
		else if ((strlen(argv[a]) == strlen(cline[i].nev)) && (cline[i].argc == 2) && (argc - a >= 3))
		{
		    argv[a] = NULL;
		    a++;
		    aa++;
		    parm = argv[a];
		    argv[a] = NULL;
		    a++;
		    aa++;
		    parm2 = argv[a];
		    argv[a] = NULL;
		    aa++;
		    ok = 1;
		    break;
		}
	    }
	}
	if (ok)
	{
#ifdef DEBUG
	    debug(NULL, "read_commandline: cline=\"%s\", parm=\"%s\"\n", cline[i].nev, parm);
#endif
	    if (parm2)
	    {
		ret = (*cline[i].func) (argv[0], parm, parm2);
		ret2 = 1;
	    }
	    else
	    {
		ret = (*cline[i].func) (argv[0], parm);
	    }
	    if (ret == 1)
	    {
		break;
	    }
	}
	else if (*argv[a] == '-')
	{
	    kiir(NULL, "Hibasan megadott opcio: \"%s\", javaslom, nezz helpet (%s --help)\r\n", argv[a], argv[0]);
	    return 1;
	}
    }
    if (!ret && (argc <= aa))
    {
	if (!ret2)
	{
	    ret = do_help(argv[0]);
	}
    }
    
    return ret;
}

extern int do_help(char * progname)
{
#ifdef WIN32
# define USERNAME "USERNAME"
#else
# define USERNAME "USER"
#endif
    kiir(NULL,
	    "Hasznalat: %s [<opciok>] [<fordulok>]\r\n\r\n"
	    "  -c FILE, --config=FILE   FILE hasznalata configuracios filekent\r\n"
            "                           (default a " CONF_FILE ")\r\n"
	    "  -o DIR,  --out=DIR       DIR-be kerulnek a reformatalt fordulok.\r\n"
	    "                           Ha DIR=stdout, a standard outputra mennek.\r\n"
	    "  -p,      --pdirconf      conffilekent csak program directoryban levo lesz\r\n"
	    "                           hasznalva (ha ott nincs cfg, akkor semmi!)\r\n"
	    "  -s,      --set direktiva=ertek\r\n"
	    "                           kozvetlen config beallitas\r\n"
	    "           --cache build karszam\r\n"
	    "                           felepiti az adott karakter cahce-et\r\n"
#ifdef WIN32
	    "           --nopause       futas utan nem var billentyulenyomast\r\n"
#endif
	    "  -q,      --quiet         nema uzem (csak reformatal, nem ir ki semmit)\r\n"
	    "  -h,      --help,         help kiirasa\r\n"
	    "  -v,      --version       version kiirasa\r\n"
	    "\r\n"
	    "  %s user configjai:  %s\r\n"
	    "\r\n"
	    , progname, getenv(USERNAME), bconf_get_value("confdir", conftype_user));
    
    return 1;
}

extern int do_config(char * progname, char * conffile)
{
    bconf_set_value("conffile", conftype_commandline, conffile);
    return 0;
}

extern int do_set(char * progname, char * setting)
{
    char * p1;
    char * p2;
    
    if (p1 = strstr(setting, "="))
    {
	*p1 = '\0';
	p1++;
	if (!isdigit2(*p1))
	{
	    if (p2 = malloc(strlen(p1)+3))
	    {
		sprintf(p2, "\"%s\"", p1);
		conf_load_str("commandline", setting, p2);
		free(p2);
		bconf_set_record_from_curr(conftype_commandline, confset_commandline);
	    }
	}
	else
	{
	    conf_load_value("commandline", setting, p1, confset_commandline, NULL);
	}
    }
    return 0;
}

extern int do_output(char * progname, char * out)
{
    if (strcmp(out, "stdout") == 0)
    {
	dataflow_set(fordulo_output_standard);
    }
    else
    {
	bconf_set_value("outdir", conftype_commandline, out);
    }
    return 0;
}

extern int do_input(char * progname, char * in)
{
    if (strcmp(in, "stdin") == 0)
    {
    }
    else
    {
	bconf_set_value("indir", conftype_commandline, in);
    }
    return 0;
}

extern int do_version(char * progname)
{
//    kiir(NULL, "%s\r\n", VERSION);
    kiir(NULL, "");
    return 1;
}

extern int do_quiet(char * progname)
{
    set_quiet(1);
    return 0;
}

extern int do_pdirconf(char * progname)
{
    set_pdirconf(1);
    return 0;
}

extern int do_psziernyo(char * progname)
{
    set_pszi_mode(1);
    return 0;
}

#ifdef WIN32
extern int do_nopause(char * progname)
{
    set_pdirconf(1);
    return 0;
}
#endif
