/*
 * Copyright (C) 2003  Vizi Gabor (vizi@csucs.net)
 *
 * Ez az adatallomany a reformat program forraskodjanak resze.
 *
 * A program hasznalata, masolasa, terjesztese maganszemely 
 * bevetelt nem eredmenyezo celjara az eredeti forras, 
 * a szerzoi jogi megjegyzes es a hasznalati feltetelek 
 * feltuntetesevel es valtozatlanul hagyasaval, szabadon lehetseges. 
 * A program minden garancia nelkul kerul kozreadasra, a program
 * mukodesebol fakado osszes kockazat a felhasznalot terheli. 
 */
#define F_MEZO_INTERNAL_ACCESS
#include "include.h"
#include "f_fordulo.h"
#ifndef JUST_NEED_TYPES
# define JUST_NEED_TYPES
# include "fordulo.h"
# include "format.h"
# undef JUST_NEED_TYPES
#else
# include "fordulo.h"
# include "format.h"
#endif
#include "globals.h"
#include "output.h"
#include "f_csata.h"

static int enemy_aname = 0;

extern int mezo_store(t_fordulo * fordulo, int x, int y)
{
    t_mezo * mezo;
    
    if (aconf_get_curr("korvege_mezo_kiiras") != 1)
    {
	return 0;
    }
    
    if (!fordulo)
    {
#ifdef DEBUG
	debug(fordulo, "mezo_store: bejovo adathiba");
#endif
	return -1;
    }

    if (!(mezo = malloc(sizeof(t_mezo))))
    {
#ifdef DEBUG
	debug(fordulo, "mezo_store: ERROR nincs memoria eltarolni a mezot");
#endif
	return -1;
    }
    
    mezo->ssz = 0;
    mezo->x = x;
    mezo->y = y;
    mezo->next = NULL;
    mezo->enemy_elso = NULL;
    mezo->enemy_utolso = NULL;
    
    if (fordulo->mezo_utolso)
    {
	mezo->ssz = fordulo->mezo_utolso->ssz + 1;
	fordulo->mezo_utolso->next = mezo;
    }
    fordulo->mezo_utolso = mezo;
    if (!fordulo->mezo_elso)
    {
	fordulo->mezo_elso = mezo;
    }
    
#ifdef DEBUG
	debug(fordulo, "mezo_store: [%d,%d]\n", mezo->x, mezo->y);
#endif
    return 1;
}

extern int mezo_store_delete(t_fordulo * fordulo)
{
    t_mezo * mezo;
    t_mezo * next;
    
    for (mezo = fordulo->mezo_elso; mezo; mezo = next)
    {
	next = mezo->next;
	if (mezo->enemy_elso)
	{
	    enemy_store_delete(mezo);
	}
	free(mezo);
    }
    enemy_aname = 0;
    fordulo->mezo_elso = NULL;
    fordulo->mezo_utolso = NULL;
}

extern int mezo_kiir(t_fordulo * fordulo)
{
    t_mezo * mezo;
    int i;
    
    if (aconf_get_curr("korvege_mezo_kiiras") != 1)
    {
	return 0;
    }
    
    if (!fordulo)
    {
	return -1;
    }
    if (!fordulo->mezo_elso)
    {
	return 0;
    }
    output(fordulo,"<BR><BR>\r\n");
    output(fordulo,"<P><TABLE WIDTH=100%% CELLPADDING=1 CELLSPACING=0 BORDER=0 CLASS=\"t0\"><TR><TD><TABLE CELLPADDING=2 CELLSPACING=0 BORDER=0 CLASS=\"t1\" WIDTH=100%%>\r\n<TR><TD CLASS=\"c1\" COLSPAN=2>%s</td></tr>\r\n", (fordulo->tipus==fordulo_tipus_olimpia?(fordulo->mezo_type==mezo_type_oli_hely?"HELYEZETTEK":"ELLENFELEK"):"BEJRT MEZK"));

    for (i = 0, mezo = fordulo->mezo_elso; mezo; mezo = mezo->next)
    {
	if (mezo->x && mezo->y)
	{
	    output(fordulo, "<tr valign=\"top\"><td nowrap class=\"t1\" width=6%% align=middle> <a class=n href=\"#mezo%d\">[%d,%d]<a>&nbsp;&nbsp;&nbsp;</td>", mezo->ssz, mezo->x, mezo->y);
	}
	else
	{
	    output(fordulo, "<tr valign=\"top\"><td nowrap class=\"t1\" width=6%% align=middle> &nbsp;&nbsp;&nbsp;</td>");
	}
	output(fordulo, "<td class=\"t1\" width=94%%>");
	if (mezo->enemy_elso)
	{
	    enemy_kiir(fordulo, mezo);
	}
	output(fordulo, "&nbsp;</td>");
	output(fordulo, "</tr>\r\n");
	i++;
    }
    output(fordulo,"</table></td></tr></table><br><br>\r\n");
}

extern int enemy_store(t_fordulo * fordulo, t_mezo * mezo, int ssz, char * nev)
{
    t_enemy * enemy;
    char * nev2;
    char * s;
    
    if (aconf_get_curr("korvege_mezo_kiiras") != 1)
    {
	return 0;
    }
    
    if (!fordulo || !mezo)
    {
#ifdef DEBUG
	debug(fordulo, "enemy_store: bejovo adathiba\n");
#endif
	return -1;
    }

    if (!(enemy = malloc(sizeof(t_enemy))) || !(nev2 = strdup(nev)))
    {
#ifdef DEBUG
	debug(fordulo, "enemy_store: ERROR nincs memoria eltarolni a enemyt");
#endif
	return -1;
    }
    
    for (s = nev2 + strlen(nev2) - 1; *s == '.'; *s = '\0', s--);
    
    enemy->ssz = ssz;
    enemy->nev = nev2;
    enemy->next = NULL;
    enemy->megj = NULL;
    enemy->csata = NULL;
    enemy->aname = ++enemy_aname;
    
    if (mezo->enemy_utolso)
    {
	mezo->enemy_utolso->next = enemy;
    }
    mezo->enemy_utolso = enemy;
    if (!mezo->enemy_elso)
    {
	mezo->enemy_elso = enemy;
    }
    
#ifdef DEBUG
	debug(fordulo, "enemy_store: %d, %s\n", enemy->ssz, enemy->nev);
#endif
    return enemy_aname;
}

extern int enemy_megj_set(t_enemy * enemy, char * megj)
{
    if (!enemy)
    {
	return -1;
    }
    if (enemy->megj)
    {
	free(enemy);
    }
#ifdef DEBUG
    debug(NULL, "enemy_megj_set: %s -> %s\n", enemy->nev, megj);
#endif

    if (megj)
    {
	enemy->megj = strdup(megj);
    }
    else
    {
	enemy->megj = NULL;
    }
    return 0;
}

extern int enemy_store_delete(t_mezo * mezo)
{
    t_enemy * enemy;
    t_enemy * next;
    
    if (!mezo)
    {
	return -1;
    }
    
    for (enemy = mezo->enemy_elso; enemy; enemy = next)
    {
	next = enemy->next;
	if (enemy->nev)
	{
	    free(enemy->nev);
	}
	if (enemy->megj)
	{
	    free(enemy->megj);
	}
	if (enemy->csata)
	{
	    free(enemy->csata);
	}
	free(enemy);
    }
}

extern int enemy_kiir(t_fordulo* fordulo, t_mezo * mezo)
{
    t_enemy * enemy;
    int i;
    
    if (aconf_get_curr("korvege_mezo_kiiras") != 1)
    {
	return 0;
    }
    
    if (!mezo)
    {
	return -1;
    }
    if (!mezo->enemy_elso)
    {
	return 0;
    }
    for (i = 0, enemy = mezo->enemy_elso; enemy; enemy = enemy->next, i++)
    {
	if (i > 0)
	{
	    output(fordulo, ", ");
	}
	if (enemy->ssz)
	{
	    output(fordulo, "<a class=n href=\"#enemy%d\">%s (#%d)<a>", enemy->aname, enemy->nev, enemy->ssz);
	}
	else
	{
	    output(fordulo, "<a class=n href=\"#enemy%d\">%s<a>", enemy->aname, enemy->nev);
	}
	if (enemy->megj)
	{
	    output(fordulo, " <i>(%s)</i>", enemy->megj);
	}
    }
}

extern int kalandozocsata_kiir(t_fordulo * fordulo)
{
    t_mezo * mezo;
    t_enemy * enemy;
    int i;
    char * p1;
    char * p2;
    int h;
    
    if (aconf_get_curr("korvege_kalandozocsata_kiiras") != 1)
    {
	return 0;
    }
    
    if (!fordulo)
    {
	return -1;
    }
    if (!fordulo->mezo_elso)
    {
	return 0;
    }

    h = 0;
    i = 1;
    for (mezo = fordulo->mezo_elso; mezo; mezo = mezo->next)
    {
	if (mezo->enemy_elso)
	{
	    for (enemy = mezo->enemy_elso; enemy; enemy = enemy->next)
	    {
		if (!enemy->csata)
		{
		    continue;
		}
		if (!(fordulo_tipus_olimpia & fordulo->tipus) && ((enemy->ssz < 1000) || (enemy->ssz > 9100)))
		{
		    if ((p1 = strstr(enemy->nev, "frfival")) && isspace2(*(p1-1)))
		    {
			// nem csinalunk semmit, csak ki kell kerulni az else { continue; } agat
		    }
		    else if ((p1 = strstr(enemy->nev, "nvel")) && isspace2(*(p1-1)))
		    {
			// nem csinalunk semmit, csak ki kell kerulni az else { continue; } agat
		    }
		    else
		    {
			continue;
		    }
		}
		if (!h)
		{
		    output(fordulo,"<BR><BR>\r\n");
		    output(fordulo,"<P><TABLE WIDTH=100%% CELLPADDING=1 CELLSPACING=0 BORDER=0 CLASS=\"t0\"><TR><TD><TABLE CELLPADDING=2 CELLSPACING=0 BORDER=0 CLASS=\"t1\" WIDTH=100%%>\r\n<TR><TD CLASS=\"c1\" COLSPAN=4>%s</td></tr>\r\n",(fordulo->tipus&fordulo_tipus_olimpia?"ELLENFELEK":"KALANDOZCSATK"));
		    h = 1;
		}
		if (enemy->ssz)
		{
		    output(fordulo, "<tr valign=\"top\"><td class=\"t1\" width=6%% align=middle> %d.) </td><td nowrap class=\"t1\" %s width=15%%> <a class=n href=\"#enemy%d\">%s (#%d)<a>: ", i, (fordulo_tipus_olimpia&fordulo->tipus?"":"style=\"FONT-SIZE: 15px;\""),enemy->aname, enemy->nev, enemy->ssz);
		}
		else
		{
		    output(fordulo, "<tr valign=\"top\"><td class=\"t1\" width=6%% align=middle> %d.) </td><td nowrap class=\"t1\" %s width=15%%> <a class=n href=\"#enemy%d\">%s<a>: ", i, (fordulo_tipus_olimpia&fordulo->tipus?"":"style=\"FONT-SIZE: 15px;\""), enemy->aname, enemy->nev);
		}
		if ((fordulo_tipus_olimpia & fordulo->tipus) && (enemy->megj))
		{
		    output(fordulo, " <i>(%s)</i>", enemy->megj);
		}
		output(fordulo,"</td><td nowrap class=\"t1\" %s width=10%%> &nbsp;&nbsp;&nbsp; <b><SPAN CLASS=\"sebez\">%u</SPAN> / <SPAN CLASS=\"szenved\">%u</SPAN></b> </td>", (fordulo_tipus_olimpia&fordulo->tipus?"":"style=\"FONT-SIZE: 15px;\""), enemy->csata->sebzes_utes + enemy->csata->sebzes_egyeb, enemy->csata->sebzodes_utes + enemy->csata->sebzodes_egyeb);
		output(fordulo, "<td nowrap class=\"t1\" %s width=69%%>", (fordulo_tipus_olimpia&fordulo->tipus?"":"style=\"FONT-SIZE: 15px;\""));
		csata_osszesites_kiir(fordulo, enemy->csata, 1);
		if (!(fordulo_tipus_olimpia & fordulo->tipus) && (enemy->megj))
		{
		    output(fordulo, " <i>(%s)</i>", enemy->megj);
		}
		output(fordulo, "&nbsp;</td></tr>\r\n");
		i++;
	    }
	}
    }
    if (h)
    {
	output(fordulo,"</table></td></tr></table><br><br>\r\n");
    }
}

