/*
 * Copyright (C) 2003  Vizi Gabor (vizi@csucs.net)
 *
 * Ez az adatallomany a reformat program forraskodjanak resze.
 *
 * A program hasznalata, masolasa, terjesztese maganszemely 
 * bevetelt nem eredmenyezo celjara az eredeti forras, 
 * a szerzoi jogi megjegyzes es a hasznalati feltetelek 
 * feltuntetesevel es valtozatlanul hagyasaval, szabadon lehetseges. 
 * A program minden garancia nelkul kerul kozreadasra, a program
 * mukodesebol fakado osszes kockazat a felhasznalot terheli. 
 */
#define INPUT_INTERNAL_ACCESS
#include "include.h"
#include "input.h"
#include "dataflow.h"
#include "globals.h"
#include "win.h"
#include "conf.h"

extern char * input(t_forras * forras)
{
    struct stat sfile;
    FILE * fp = NULL;
    unsigned long size;
    char * fordulostr = NULL;
    int hiba = 0;

#ifdef DEBUG
    debug(NULL, "input 0: name= \"%s\", type=%d\n", forras->name, forras->type);
#endif
    if (forras->type&forras_type_file)
    {
	if (forras->name && (stat(forras->name, &sfile) == 0))
	{
#ifdef DEBUG
	    debug(NULL, "input 1\n");
#endif
	    size = (unsigned long)sfile.st_size;
	    if (fordulostr = malloc(size+1))
	    {
		memset(fordulostr,0,size+1);
#ifdef DEBUG
		debug(NULL, "input 3\n");
#endif
		if (fp = fopen(forras->name,"rb"))
		{
#ifdef DEBUG
		    debug(NULL, "input 4\n");
#endif
		    if(fread(fordulostr,1,size,fp))
		    {
#ifdef DEBUG
			debug(NULL, "input 5\n");
#endif
			fclose(fp);
			fordulostr[size] = '\0';
		    }
		    else
		    {
			hiba = 1;
		    }
		}
		else
		{
		    hiba = 1;
		}
	    }
	    else
	    {
		hiba = 1;
	    }
	}
	else
	{
	    hiba = 1;
	}
    }
    
    if (hiba)
    {
	if (fordulostr)
	{
	    free(fordulostr);
	    fordulostr = NULL;
	}
    }
    return fordulostr;
}

extern t_forras * forras_create(char * name, t_forras_type type)
{
    t_forras * temp = NULL;
    char * indir;
    char * s;
    
    if (!name)
    {
	return NULL;
    }
    
    indir = bconf_get_indir();
    if ((name[0] != '/') && (name[0] != '\\'))
    {
	if (!(s = get_filename(indir, name)))
	{
	    return NULL;
	}
    }
    else if (!(s = strdup(name)))
    {
	return NULL;
    }
    
    if (!(temp = malloc(sizeof(t_forras))))
    {
	free(s);
	return NULL;
    }
    
    temp->name = s;
    temp->type = type;
    return temp;
}

extern char * forras_indir(t_forras * forras)
{
    char * ret = NULL;
    char * p1;
    char sep;
    char tmp;
    
#ifndef WIN32
    sep = '/';
#else
    sep = '\\';
#endif
    
    if (p1 = strrchr(forras->name, sep))
    {
	tmp = *p1;
	*p1 = '\0';
	ret = strdup(forras->name);
	*p1 = tmp;
    }
    
    return ret;
}

extern int forras_destroy(t_forras * forras)
{
    if (forras)
    {
	if (forras->name)
	{
	    free(forras->name);
	    forras->name = NULL;
	}
	free(forras);
    }
}

extern int dir_list(char * path, char * mask, char *** result)
{
#ifdef WIN32
    int n;
    
    n = wdir(path, mask, result);
    
#else
    struct dirent **namelist;
    char * name;
    char ** res;
    int i, n;
    int hiba = 0;

    namelist = NULL;
    
    n = scandir(path, &namelist, NULL, alphasort);
    
    if (n > 0)
    {
	if (!(res = malloc(sizeof(char *) * n)))
	{
	    return -1;
	}
	memset(res, 0, sizeof(char *) * n);
	
	for (i = 0; namelist[i] && namelist[i]->d_name && (i < n); i++)
	{
	    if (!(name = strdup(namelist[i]->d_name)))
	    {
		hiba = 1;
		break;
	    }
	    
	    res[i] = name;
	}
    }
    
    for(i = 0; i < n; i++)
    {
	if (namelist && namelist[i])
	{
	    free(namelist[i]);
	    namelist[i] = NULL;
	}
	if (hiba)
	{
	    if (res && res[i])
	    {
		free(res[i]);
	    }
	}
    }
    if (namelist)
    {
        free(namelist);
	namelist = NULL;
    }
    if (hiba)
    {
	if (res)
	{
	    free(res);
	}
	res = NULL;
	n = -1;
    }

    *result = res;
#endif
    return n;
}
