/*
 * Copyright (C) 2003-2005  Vizi Gabor (vizi@csucs.net)
 *
 * Ez az adatallomany a reformat program forraskodjanak resze.
 *
 * A program hasznalata, masolasa, terjesztese maganszemely 
 * bevetelt nem eredmenyezo celjara az eredeti forras, 
 * a szerzoi jogi megjegyzes es a hasznalati feltetelek 
 * feltuntetesevel es valtozatlanul hagyasaval, szabadon lehetseges. 
 * A program minden garancia nelkul kerul kozreadasra, a program
 * mukodesebol fakado osszes kockazat a felhasznalot terheli. 
 */
#define KARAKTER_INTERNAL_ACCESS
#include "include.h"
#include "karakter.h"
#include "fordulo.h"
#include "spell.h"
#include "targy.h"
#include "fegyver.h"
#include "sebzes.h"
#include "cache.h"

static t_varazslatsebzes varazslatsebzes[] =
{
    {836, 20, 200, 0, 0, 0, targy_flag_none},		// Sshreakza emulett
    {890, 10, 50, 200, 200, 0, targy_flag_none},	// agvari manafokusz
    {956, 10, 50, 200, 200, 0, targy_flag_none},	// xanathal karkoto
    {944, 10, 50, 200, 200, 0, targy_flag_none},	// eben talizman
    {1135, 5, 200, 0, 0, 0, targy_flag_none}		// invazios amulett
};

static char * bonuszok_neve[] =
{
    "er",
    "iq",
    "gyessg",
    "egszsg",
    "szerencse",
    "maxtu",
    "maxp",
    "maxvp",
    "maxpp",
    "vdekezs",
    "tmads",
    "sebzs",
    "vp regenerci",
    NULL
};

static char * kepessegek_neve[] =
{
    "rejtzkds",
    "nyomkvets",
    "lops",
    "mszs",
    "csapdakszits",
    "csapdaszlels",
    "gygyits",
    "titkosirs",
    "felderits",
    "szrnyidomits",
    "teolgia",
    "taumaturgia",
    "szerencsejtk",
    "versengs",
    "harcmvszet",
    "szkander",
    "zene",
    "szrnyismeret",
    "pszi",
    "zrnyits",
    "vadszat",
    "vitalits",
    "bnyszat",
    "testpits",
    "szs",
    "ordits",
    "fzs",
    "steolgia",
    "staumaturgia",
    NULL
};

extern int kepesseg_id_from_hyper_id(int id)
{
    int ret;
    
    if ((id < 1) || (id > 23))
    {
	return 0;
    }

    switch (id)
    {
	case 13:
	    ret = 19;
	    break;
	case 14:
	case 15:
	case 16:
	    ret = id - 1;
	    break;
	case 19:
	case 20:
	case 21:
	case 22:
	    ret = id + 1;
	    break;
	case 23:
	    ret = id + 2;
	default:
	    ret = id;
    }
    
    return ret;
}

extern int karakter_init(t_fordulo * fordulo)
{
    t_karakter * karakter;
    t_varazslatsebzes * v;
    
    if (!fordulo)
    {
	return 0;
    }
    
    if (karakter = malloc(sizeof(t_karakter)))
    {
        memset(karakter,0,sizeof(t_karakter));
	if (v = malloc(sizeof(varazslatsebzes)))
	{
	    memcpy(v, &varazslatsebzes, sizeof(varazslatsebzes));
	    karakter->varazslatsebzes = v;
	    karakter->varazslatsebzes_size = sizeof(varazslatsebzes)/sizeof(t_varazslatsebzes);
	}
	karakter->fordulo = fordulo;
        fordulo->karakter = karakter;
	return 1;
    }
    
    return 0;
}

extern int karakter_deinit(t_fordulo * fordulo)
{
    int vi;
    t_varazslatsebzes * varazslatsebzes;
    
    if (!fordulo || !fordulo->karakter)
    {
	return 0;
    }
    
    karakter_fegyverlist_destroy(fordulo->karakter);
    karakter_spelllist_destroy(fordulo->karakter);
    karakter_targylist_destroy(fordulo->karakter);
    karakter_targy_figyelt_list_destroy(fordulo->karakter);
    if(fordulo->karakter->varazslatsebzes)
    {
	free(fordulo->karakter->varazslatsebzes);
    }
    fordulo->karakter->varazslatsebzes = NULL;
    fordulo->karakter->varazslatsebzes_size = 0;
    return 0;
}

extern char * get_bonuszok_neve(int i)
{
    if (i < (sizeof(bonuszok_neve)/sizeof(char*)))
    {
	return bonuszok_neve[i];
    }
    return NULL;
}

extern char * get_kepessegek_neve(int i)
{
    if (i < (sizeof(kepessegek_neve)/sizeof(char*)))
    {
	return kepessegek_neve[i];
    }
    return NULL;
}

extern int karakter_targy_figyelt_init(t_karakter * karakter)
{
    t_targy * targy;
    t_targy_figyelt * figyelt;
    t_targy_figyelt * figyelt_k;
    int ret = 0;
    
    if (!karakter || karakter->karakter_targy_figyelt_list_head)
    {
	return 0;
    }
    
    for (figyelt = get_targy_figyelt_head(); figyelt; figyelt = figyelt->next)
    {
        if (figyelt_k = targy_figyelt_create(figyelt->ssz, figyelt->confset, figyelt->min, figyelt->max))
        {
	    figyelt_k->targy = get_targy_from_ssz(figyelt_k->ssz);
#ifdef DEBUG
	    debug(NULL, "karakter_targy_figyelt_init: #%d, targy=%p, min=%d, max=%d\n", figyelt_k->ssz, figyelt_k->targy, figyelt->min, figyelt->max);
#endif
	    if (!karakter->karakter_targy_figyelt_list_head)
	    {
	        karakter->karakter_targy_figyelt_list_head = figyelt_k;
	    }
	    if (karakter->karakter_targy_figyelt_list_tail)
	    {
	        karakter->karakter_targy_figyelt_list_tail->next = figyelt_k;
	    }
	    karakter->karakter_targy_figyelt_list_tail = figyelt_k;
	}
    }
    return ret;
}

extern int karakter_targy_figyelt_add(t_karakter * karakter, int ssz, int min, int max)
{
    t_targy_figyelt * temp;
    t_targy_figyelt * curr;
    t_targy_figyelt * prev;
    int ok;
    int ret;

    if (!karakter)
    {
	return -1;
    }
    
    if (!karakter->karakter_targy_figyelt_list_head)
    {
	karakter_targy_figyelt_init(karakter);
    }
    
    if (!(temp = targy_figyelt_create(ssz, confset_kar, min, max)))
    {
	return -1;
    }
    temp->targy = get_targy_from_ssz(temp->ssz);
    
    for (ok = 0, prev = NULL, curr = karakter->karakter_targy_figyelt_list_head; curr; prev = curr, curr = curr->next)
    {
	if (curr->ssz == ssz)
	{
	    ok = 1;
	    break;
	}
    }
    
    if (ok)
    {
	temp->next = curr->next;
	if (prev)
	{
	    prev->next = temp;
	}
	else
	{
	    karakter->karakter_targy_figyelt_list_head = temp;
	}
	if (curr == karakter->karakter_targy_figyelt_list_tail)
	{
	    karakter->karakter_targy_figyelt_list_tail = temp;
	}
	free(curr);
    }
    else
    {
	if (!karakter->karakter_targy_figyelt_list_head)
	{
	    karakter->karakter_targy_figyelt_list_head = temp;
	}
	if (karakter->karakter_targy_figyelt_list_tail)
	{
	    karakter->karakter_targy_figyelt_list_tail->next = temp;
	}
	karakter->karakter_targy_figyelt_list_tail = temp;
    }
#ifdef DEBUG
    debug(NULL, "karakter_targy_figyelt_add: #%d confset=%d\n", temp->ssz, temp->confset);
#endif
    return 1;
}

extern t_targy_figyelt * karakter_targy_figyelt_check(t_karakter * karakter, int ssz)
{
    t_targy_figyelt * temp;
    t_targy_figyelt * ret = NULL;
    
    if (!karakter)
    {
	return 0;
    }
    
    for (temp = karakter->karakter_targy_figyelt_list_head; temp; temp = temp->next)
    {
	if (temp->ssz == ssz)
	{
	    ret = temp;
	    break;
	}
    }

    return ret;
}

extern int karakter_targy_figyelt_set_targylist(t_karakter * karakter, int ssz, t_karakter_targylist * karakter_targylist)
{
    t_targy_figyelt * temp;
    int ret = 0;
    
    if (!karakter)
    {
	return 0;
    }
    
    for (temp = karakter->karakter_targy_figyelt_list_head; temp; temp = temp->next)
    {
	if (temp->ssz == ssz)
	{
	    temp->targylist = karakter_targylist;
	    ret = ssz;
	    break;
	}
    }

    return ret;
}

extern int karakter_targy_figyelt_list_destroy(t_karakter * karakter)
{
    t_targy_figyelt * figyelt;
    t_targy_figyelt * next;
    
    for (figyelt = karakter->karakter_targy_figyelt_list_head; figyelt; figyelt = next)
    {
	next = figyelt->next;
	free(figyelt);
    }
    karakter->karakter_targy_figyelt_list_head = NULL;
    karakter->karakter_targy_figyelt_list_tail = NULL;
}

extern int karakter_targylist_head_init(t_karakter * karakter)
{
    if (karakter->karakter_targylist_head || karakter->karakter_targylist_tail)
    {
	karakter_targylist_destroy(karakter);
    }
}

extern t_karakter_targylist * get_karakter_targylist_head(t_karakter * karakter, int i)
{
    t_karakter_targylist ** t;
    
    t = &karakter->karakter_targylist_head;
    
    return t[2*i];
}

extern int karakter_targylist_destroy(t_karakter * karakter)
{
    t_karakter_targylist * temp;
    t_karakter_targylist * next;
    t_karakter_targylist ** t;
    int i;
    
    for (i = 0; i <= KARAKTER_TARGYLIST_NUM; i++)
    {
	t = &karakter->karakter_targylist_head;
	for (temp = t[2*i]; temp; temp = next)
	{
	    next = temp->next;
	    free(temp);
	}

	t[2*i] = NULL;
	t[2*i+1] = NULL;
    }
    
    return 0;
}

int karakter_targylist_add_elem_sort(t_karakter * karakter, t_targy * targy, int darab, t_targy_flag flag, t_targy_hol hol, int i)
{
    t_karakter_targylist ** t;
    t_karakter_targylist * temp;
    t_karakter_targylist * curr;
    int ok;
    int diff;
    t_targy_figyelt * targy_figyelt;
    
    if (!targy)
    {
	return 0;
    }
    
    if (!(temp = malloc(sizeof(t_karakter_targylist))))
    {
#ifdef DEBUG
        debug(karakter->fordulo, "karakter_targylist_add_elem_sort: could not allocate memory for temp\n");
#endif
	return -1;
    }
    
    memset(temp,0,sizeof(t_karakter_targylist));
    
    temp->ssz = targy->ssz;
    temp->targy = targy;
    temp->darab = darab;
    if (hol&targy_hol_nalad)
    {
	temp->nalad = darab;
    }
    if (hol&targy_hol_bankban)
    {
	temp->bankban = darab;
    }
    temp->flag = flag;
    temp->valtozas_checked = 0;
    t = &karakter->karakter_targylist_head;
    
#ifdef DEBUG
    debug(karakter->fordulo, "karakter_targylist_add_elem_sort: ssz=%d, targy=\"%s\", darab=%d (%d, %d), flag=%d, hol=%d, i=%d\n", temp->ssz, temp->targy->nev, temp->darab, temp->nalad, temp->bankban, temp->flag, hol, i);
#endif
    
    ok = 1;
    if (!t[2*i])
    {
	t[2*i] = temp;
    }
    else
    {
	if (strcmp_(temp->targy->nev, t[2*i]->targy->nev) < 0)
	{
	    temp->next = t[2*i];
	    t[2*i] = temp;
	}
	else
	{
	    ok = 0;
	    for (curr = t[2*i]; curr->next; curr = curr->next)
    	    {
		diff = strcmp_(temp->targy->nev, curr->next->targy->nev);
		if ((diff == 0) && (strlen(temp->targy->nev) == strlen(curr->next->targy->nev)))
		{
		    curr = curr->next;
#ifdef DEBUG
		    debug(karakter->fordulo, "karakter_targylist_add_elem_sort: curr: %d (%d, %d), temp: %d (%d, %d)\n", curr->darab, curr->nalad, curr->bankban, temp->darab, temp->nalad, temp->bankban);
#endif
		    curr->darab += temp->darab;
		    curr->nalad += temp->nalad;
		    curr->bankban += temp->bankban;
		    free(temp);
		    return 1;
		}
		else if (diff < 0)
		{
		    temp->next = curr->next;
		    curr->next = temp;
		    ok = 1;
		    break;
		}
	    }
	}
    }
    
    if (!t[2*i+1] || !ok)
    {
	if (t[2*i+1])
	{
	    t[2*i+1]->next = temp;
	}
	t[2*i+1] = temp;
    }

    if ((i == 0) && (targy_figyelt = karakter_targy_figyelt_check(karakter, temp->ssz)))
    {
//	temp->figyelt = targy_figyelt;
	karakter_targy_figyelt_set_targylist(karakter, targy->ssz, temp);
    }

    return 1;
}

extern int karakter_targylist_add(t_karakter * karakter, t_targy * targy, int darab, t_targy_flag flag, t_targy_hol hol)
{
    t_varazslatsebzes * v;
    int i;
    
    if ((i = karakter_targylist_add_elem_sort(karakter, targy, darab, flag, hol, 0)) != 1)
    {
	return i;
    }
    if ((targy->ssz == 893) || (targy->ssz == 894) || (targy->ssz == 895))
    {
	if (flag&targy_flag_kf)
	{
	    karakter->tleikan_targy_darab++;
	}
    }
    else if ((flag&targy_flag_kf) && ((targy->ssz == 987) || (targy->ssz == 996) || (targy->ssz == 1005) || (targy->ssz == 1014) || (targy->ssz == 1023) || (targy->ssz == 1032) || (targy->ssz == 1041) || (targy->ssz == 1050) || (targy->ssz == 1059)))
    {
	karakter->faji_kopeny = 1;
    }
    else if ((flag&targy_flag_kf) && (targy->ssz == 746))
    {
	karakter->ovsal = 1;
    }
    else if (targy->ssz == 795)
    {
	karakter->gharitonfokusz = 1;
    }
    else if ((flag&targy_flag_kf) && ((targy->ssz == 307) || (targy->ssz == 308) || (targy->ssz == 309)))
    {
	karakter->papikarkoto = (targy->ssz - 306) * 2 - 1;
    }
    
    for (i = 0, v = karakter->varazslatsebzes; i < karakter->varazslatsebzes_size; i++)
    {
	if (targy->ssz == v[i].ssz)
	{
	    v[i].van = (aconf_get_curr("spell_sebzes_bonusz_csak_meglevo") == 0) || (aconf_get_curr("spell_sebzes_bonusz_csak_meglevo") == 1) || ((aconf_get_curr("spell_sebzes_bonusz_csak_meglevo") == 2) && ((flag&targy_flag_kf) || targy->nonkf));
	    v[i].flag = flag;
	    v[i].targy = targy;
#ifdef DEBUG
	    debug(karakter->fordulo, "karakter_targylist_add: (#%d) varazslatsebzes %+d%%, de max %d, van=%d\n", v[i].ssz, v[i].bonusz, v[i].max, v[i].van);
#endif
	}
    }
    
    if (targy->fegyver)
    {
	karakter_fegyverlist_add_fegyver(karakter, targy);
    }
    if (targy->bonuszok)
    {
	if (targy->ssz != 454)
	{
	    karakter_targylist_add_elem_sort(karakter, targy, darab, flag, hol, 1);
	}
    }
    if (targy->kepessegek)
    {
	karakter_targylist_add_elem_sort(karakter, targy, darab, flag, hol, 2);
    }
    if (targy->spell)
    {
	karakter_targylist_add_elem_sort(karakter, targy, darab, flag, hol, 3);
    }
    
    return 1;
}

extern int karakter_targylist_init(t_karakter * karakter)
{
    karakter_targylist_destroy(karakter);
    karakter_fegyverlist_head_init(karakter);
    karakter_targylista_figyelt_init(karakter);
}

extern int karakter_spelllist_head_init(t_karakter * karakter)
{
    if (karakter->karakter_spelllist_head || karakter->karakter_spelllist_tail)
    {
	karakter_spelllist_destroy(karakter);
    }
}

extern t_karakter_spelllist * get_karakter_spelllist_head(t_karakter * karakter, int i)
{
    t_karakter_spelllist ** t;
    
    t = &karakter->karakter_spelllist_head;
    
    return t[2*i];
}

extern int karakter_spelllist_destroy(t_karakter * karakter)
{
    t_karakter_spelllist * temp;
    t_karakter_spelllist * next;
    t_karakter_spelllist ** t;
    int i;
    
    for (i = 0; i <= 1; i++)
    {
	t = &karakter->karakter_spelllist_head;
	for (temp = t[2*i]; temp; temp = next)
	{
	    next = temp->next;
	    free(temp);
	}

	t[2*i] = NULL;
	t[2*i+1] = NULL;
    }
    
    return 0;
}

int karakter_spelllist_add_elem(t_karakter * karakter, t_spell * spell, int i)
{
    t_karakter_spelllist ** t;
    t_karakter_spelllist * temp;
    t_karakter_spelllist * curr;
    int mar_van;
    
    if (!spell)
    {
	return 0;
    }
    if (!(temp = malloc(sizeof(t_karakter_spelllist))))
    {
#ifdef DEBUG
        debug(karakter->fordulo, "karakter_spelllist_add_elem: could not allocate memory for temp\n");
#endif
	return -1;
    }
    
    memset(temp,0,sizeof(t_karakter_spelllist));
    
    temp->ssz = spell->ssz;
    temp->spell = spell;

    t = &karakter->karakter_spelllist_head;

    for (mar_van = 0, curr = t[2*i]; curr; curr = curr->next)
    {
	if (curr->ssz == temp->ssz)
	{
	    mar_van = 1;
	    break;
	}
    }

    if (!mar_van)
    {
	if (!t[2*i])
	{
	    t[2*i] = temp;
	}
	if (t[2*i+1])
	{
	    t[2*i+1]->next = temp;
	}
	t[2*i+1] = temp;

#ifdef DEBUG
	debug(karakter->fordulo, "karakter_spelllist_add_elem: ssz=%d, spell=\"%s\", i=%d\n", temp->ssz, temp->spell->nev, i);
#endif

	if (i == 0)
	{
	    spell_figyelt_set_spelllist(spell->ssz, temp);
	}
    }
    else
    {
	free(temp);
    }
    return 1;
}

extern int karakter_spelllist_add(t_karakter * karakter, t_spell * spell)
{
    karakter_spelllist_add_elem(karakter, spell, 0);

    if (sebzes_calculate(spell->sebzes))
    {
	karakter_spelllist_add_elem(karakter, spell, 1);
    }
}

extern int karakter_spelllist_init(t_karakter * karakter)
{
    karakter_spelllist_destroy(karakter);
}

extern int karakter_spelllist_init_fentvan(t_karakter * karakter)
{
    t_karakter_spelllist * temp;
    
    if (!karakter)
    {
	return -1;
    }
    
    for (temp = karakter->karakter_spelllist_head; temp; temp = temp->next)
    {
	temp->fentvan = 0;
    }
    
    return 0;
}

extern int karakter_spelllist_set_fentvan(t_karakter * karakter, int ssz)
{
    t_karakter_spelllist * temp;
    int ret = -1;
    
    if (!karakter)
    {
	return -1;
    }
    
    for (temp = karakter->karakter_spelllist_head; temp; temp = temp->next)
    {
	if (temp->ssz == ssz)
	{
	    temp->fentvan = 1;
	    ret = ssz; 
	    break;
	}
    }
    
    spell_figyelt_set_fentvan(ssz);
    
    return ret;
}

extern int karakter_spelllist_check_spell(t_karakter * karakter, int ssz)
{
    t_karakter_spelllist * temp;
    int ret = 0;
    
    if (!karakter)
    {
	return 0;
    }
    
    for (temp = karakter->karakter_spelllist_head; temp; temp = temp->next)
    {
	if (temp->ssz == ssz)
	{
	    ret = temp->ssz;
	    break;
	}
    }
    
    return ret;
}

extern int karakter_targylist_check_targy_osszerakas(t_karakter * karakter, int ssz)
{
    t_targy_alkoto * alkoto;
    t_targy * targy;
    int ret = 0;
    int n, ok;
    int darab, darab_max;
    
    if (targy = get_targy_from_ssz(ssz))
    {
	for (n = 0, ok = 0, darab_max = -1, alkoto = targy->alkoto; alkoto; alkoto = alkoto->next)
	{
	    if (alkoto->darab < 1)
	    {
		continue;
	    }
	    
	    n++;
	    if ((alkoto->ssz == 50) && (ssz >= 982) && (ssz <= 1063))
	    {
		ok++;
		continue;
	    }
	    darab = karakter_targylist_check_targy_nalad(karakter, alkoto->ssz) / alkoto->darab;
	    if (darab >= 0)
	    {
		ok++;
	    	if ((darab_max == -1) || (darab < darab_max))
		{
	    	    darab_max = darab;
		}
	    }
	}
	if (n && ok && (n == ok))
	{
	    ret = (darab_max >= 0 ? darab_max : 0);
	}
    }
    
    return ret;
}

extern int karakter_targylist_check_targy(t_karakter * karakter, int ssz)
{
    t_karakter_targylist * temp;
    int ret = 0;
    
    if (!karakter)
    {
	return 0;
    }
    
    for (temp = karakter->karakter_targylist_head; temp; temp = temp->next)
    {
	if (temp->ssz == ssz)
	{
	    ret = temp->ssz;
	    break;
	}
    }
    
    return ret;
}

extern int karakter_targylist_check_targy_nalad(t_karakter * karakter, int ssz)
{
    t_karakter_targylist * temp;
    int ret = 0;
    
    if (!karakter)
    {
	return 0;
    }
    
    for (temp = karakter->karakter_targylist_head; temp; temp = temp->next)
    {
	if (temp->ssz == ssz)
	{
	    ret = temp->nalad;
	    break;
	}
    }
    
    return ret;
}

extern int karakter_targylist_check_targy_kf(t_karakter * karakter, int ssz)
{
    t_karakter_targylist * temp;
    int ret = 0;
    
    if (!karakter)
    {
	return 0;
    }
    
    for (temp = karakter->karakter_targylist_head; temp; temp = temp->next)
    {
	if (temp->ssz == ssz)
	{
	    if (temp->flag&targy_flag_kf)
	    {
		ret = temp->ssz;
	    }
	    break;
	}
    }
    
    return ret;
}

extern t_karakter_targylist * karakter_targylist_get_targy(t_karakter * karakter, int ssz)
{
    t_karakter_targylist * temp;
    t_karakter_targylist * ret = NULL;
    
    if (!karakter)
    {
	return NULL;
    }
    
    for (temp = karakter->karakter_targylist_head; temp; temp = temp->next)
    {
	if (temp->ssz == ssz)
	{
	    ret = temp;
	    break;
	}
    }
    
    return ret;
}

extern int karakter_targylista_valtozas_check(t_karakter * karakter_mostani, t_karakter * karakter_elozo)
{
    t_karakter_targylist * t1;
    t_karakter_targylist * t2;
    int kulonbseg;
    int ret = 0;
    
    if (!karakter_mostani || !karakter_elozo)
    {
	return 0;
    }
    for (t1 = karakter_mostani->karakter_targylist_head; t1; t1 = t1->next)
    {
	kulonbseg = 0;
	if (t2 = karakter_targylist_get_targy(karakter_elozo, t1->ssz))
	{
	    kulonbseg = t1->darab - t2->darab;
	    if (kulonbseg != 0)
	    {
		karakter_targylist_add_elem_sort(karakter_mostani, t1->targy, kulonbseg, targy_flag_none, targy_hol_osszesen, 4);
		ret++;
	    }
	    t2->valtozas_checked = 1;
	}
	else
	{
	    karakter_targylist_add_elem_sort(karakter_mostani, t1->targy, t1->darab, targy_flag_none, targy_hol_osszesen, 4);
	    ret++;
	}
    }
    for (t2 = karakter_elozo->karakter_targylist_head; t2; t2 = t2->next)
    {
	if (t2->valtozas_checked)
	{
	    continue;
	}
	karakter_targylist_add_elem_sort(karakter_mostani, t2->targy, (0 - t2->darab), targy_flag_none, targy_hol_osszesen, 4);
	ret++;
    }
    
    return ret;
}

extern t_karakter_fegyverlist * get_karakter_fegyverlist_head(t_karakter * karakter, int i)
{
    if (i < 7 && i >= 0)
    {
	return karakter->karakter_fegyverlist_head[i];
    }
    return NULL;
}

int karakter_fegyverlist_add_fegyver_elem(t_karakter * karakter, t_targy * targy, int i)
{
    t_karakter_fegyverlist * temp;
    t_karakter_fegyverlist * curr;
    int mar_van;
    
    if (!(temp = malloc(sizeof(t_karakter_fegyverlist))))
    {
#ifdef DEBUG
        debug(karakter->fordulo, "karakter_fegyverlist_add_fegyver_elem: could not allocate memory for temp\n");
#endif
	return -1;
    }
    
    memset(temp,0,sizeof(t_karakter_fegyverlist));
    
    temp->ssz = targy->ssz;
    temp->fegyver = targy->fegyver;
    temp->next = NULL;

    for (mar_van = 0, curr = karakter->karakter_fegyverlist_head[i]; curr; curr = curr->next)
    {
	if (curr->ssz == temp->ssz)
	{
	    mar_van = 1;
	    break;
	}
    }

    if (!mar_van)
    {
	if (!karakter->karakter_fegyverlist_head[i])
	{
	    karakter->karakter_fegyverlist_head[i] = temp;
	}
	if (karakter->karakter_fegyverlist_tail[i])
	{
	    karakter->karakter_fegyverlist_tail[i]->next = temp;
	}
	karakter->karakter_fegyverlist_tail[i] = temp;
#ifdef DEBUG
	debug(karakter->fordulo, "karakter_fegyverlist_add_fegyver_elem: %p %s (#%d) type=%d (%d), sebzes=%dD%d%+d, spell=%d\n", temp, targy->nev, targy->ssz, targy->fegyver->type, i, targy->fegyver->sebzes.darab, targy->fegyver->sebzes.ertek, targy->fegyver->sebzes.bonusz, (targy->fegyver->spell ? targy->fegyver->spell->ssz : 0));
#endif
    }
    else
    {
	free(temp);
    }
    
    return 0;
}

extern int karakter_fegyverlist_add_fegyver(t_karakter * karakter, t_targy * targy)
{
    int i;
    int r;
    
    if (!targy || !targy->fegyver)
    {
	return 0;
    }
    
    if (targy->fegyver->type & fegyver_type_okol)
    {
	i = 0;
    }
    else if (targy->fegyver->type & fegyver_type_szuro)
    {
	i = 1;
    }
    else if (targy->fegyver->type & fegyver_type_vago)
    {
	i = 2;
    }
    else if (targy->fegyver->type & fegyver_type_uto)
    {
	i = 3;
    }
    else if (targy->fegyver->type & fegyver_type_lo)
    {
	i = 4;
    }
    else if (targy->fegyver->type & fegyver_type_dobo)
    {
	i = 5;
    }
    else if (targy->fegyver->type & fegyver_type_vizi)
    {
	i = 6;
    }
    else
    {
	return 0;
    }

    karakter_fegyverlist_add_fegyver_elem(karakter, targy, i);
    
    if ((targy->fegyver->type & fegyver_type_balkez) || (targy->ssz == 28))
    {
	karakter_fegyverlist_add_fegyver_elem(karakter, targy, 6);
    }
    
    return 1;
}

extern int karakter_fegyverlist_destroy(t_karakter * karakter)
{
    t_karakter_fegyverlist * temp;
    t_karakter_fegyverlist * next;
    int i;
    
    for (i = 0; i < 7; i++)
    {
	for (temp = karakter->karakter_fegyverlist_head[i]; temp; temp = next)
	{
	    next = temp->next;
	    free(temp);
	}

	karakter->karakter_fegyverlist_head[i] = NULL;
	karakter->karakter_fegyverlist_tail[i] = NULL;
    }
    
    return 0;
}

extern int karakter_fegyverlist_head_init(t_karakter * karakter)
{
    t_targy * okol;
    
    karakter_fegyverlist_destroy(karakter);
    
    okol = get_targy_from_ssz(28);
    karakter_fegyverlist_add_fegyver(karakter, okol);
    
    return 0;
}

extern int karakter_targylista_figyelt_init(t_karakter * karakter)
{
    
}
