/*
 * Copyright (C) 2003  Vizi Gabor (vizi@csucs.net)
 *
 * Ez az adatallomany a reformat program forraskodjanak resze.
 *
 * A program hasznalata, masolasa, terjesztese maganszemely 
 * bevetelt nem eredmenyezo celjara az eredeti forras, 
 * a szerzoi jogi megjegyzes es a hasznalati feltetelek 
 * feltuntetesevel es valtozatlanul hagyasaval, szabadon lehetseges. 
 * A program minden garancia nelkul kerul kozreadasra, a program
 * mukodesebol fakado osszes kockazat a felhasznalot terheli. 
 */
#define OUTPUT_INTERNAL_ACCESS
#include "include.h"
#include "output.h"
#include "fordulo.h"
#include "karakter.h"
#include "dataflow.h"
#include "globals.h"
#include "conf.h"

static int fpn = 0;
static unsigned long fpc = 0;
static int kn = 0;

extern unsigned long int get_fpn(void)
{
    return fpn;
}

extern unsigned long int get_fpc(void)
{
    return fpc;
}

extern int output(t_fordulo * fordulo, char *line, ...)
{
    va_list va;
    FILE * fp = NULL;

    if (get_cache_mode())
    {
	return 0;
    }
    if (!fordulo || (fordulo->dataflow & fordulo_output_standard))
    {
	fp = stdout;
    }
    else if (fordulo->output.file && (fordulo->dataflow & fordulo_output_normal))
    {
	if (fordulo->output.file)
	{
	    fp = fordulo->output.file;
	}
    }
    else
    {
	return 0;
    }
    va_start(va, line);
    fpc += vfprintf(fp, line, va);
    va_end(va);

    fflush(fp);
    fpn++;

    return 1;
}

extern int kiir(t_fordulo * fordulo, char *line, ...)
{
    va_list va;
    FILE * fp;

    if (get_quiet())
    {
	return 0;
    }
    if (get_cache_mode())
    {
	return 0;
    }
    
    if (!fordulo || !fordulo->kiir.file)
    {
	fp = stderr;
    }
    else
    {
	fp = fordulo->kiir.file;
    }

    if (!kn)
    {
        fprintf(fp, "%s    %s (C) Vizi Gabor [KWS]\r\n\r\n", VERSION, VST);
    }
    va_start(va, line);
    vfprintf(fp, line, va);
    va_end(va);

    fflush(fp);
    kn++;

    return 1;
}

extern int output_open(t_fordulo * fordulo)
{
    char filename[255];
    char * file = NULL;
    FILE * fp;
    char * outdir;
    char * open_flag;

    if (!fordulo || !fordulo->karakter || !fordulo->karakter->ksz)
    {
#ifdef DEBUG
	debug(fordulo, "output_open: ERROR nincs karszam vagy fordulo\n");
#endif
	return -1;
    }
    
    if (fordulo->kiir.file || fordulo->output.file)
    {
#ifdef DEBUG
	debug(fordulo, "output_open: ERROR mar van kiir vagy output\n");
#endif
	return 0;
    }

    if (fordulo->kiir.name)
    {
#ifdef DEBUG
	debug(fordulo, "output_open: open kiir\n");
#endif
	if(fp = fopen(fordulo->kiir.name ,"wb"))
	{
	    fordulo->kiir.file = fp;
	}
	else return 0;
    }
    
    strcpy(filename,"");
    open_flag = "wb";
    
    if (fordulo->dataflow & fordulo_output_standard)
    {
#ifdef DEBUG
	debug(fordulo, "output_open: open stdout\n");
#endif
	fordulo->output.file = stdout;
	return 1;
    }
    else if (fordulo->dataflow & fordulo_output_pszi)
    {
	sprintf(filename,"psziernyok.txt");
	file = get_filename(NULL, filename);
	open_flag = "ab";
    }
    else if (fordulo->dataflow & fordulo_output_normal)
    {
	outdir = bconf_get_outdir();
	if (fordulo->tipus == fordulo_tipus_olimpia)
	{
	    sprintf(filename,"%dwOLI_%.4d_%.3s.HTM", fordulo->karakter->ksz, fordulo->ev, fordulo->honap);
	}
	else
	{
	    sprintf(filename,"%dw%d.HTM", fordulo->karakter->ksz, fordulo->fsz);
	}
	if (fordulo->forras->name && strstr(fordulo->forras->name, filename))
	{
#ifdef DEBUG
	    debug(fordulo, "output_open: ERROR ugyanaz az output mint a forras\n");
#endif
	    return -2;
	}
	file = get_filename(outdir, filename);
#ifdef DEBUG
	debug(fordulo, "output_open: filename=\"%s\", file=\"%s\"\n", filename, file);
#endif
    }
    
    if (file)
    {
	fordulo->output.name = strdup(file);
    }
    if(file && (fp = fopen(file, open_flag)))
    {
#ifdef DEBUG
	debug(fordulo, "output_open: open file=\"%s\"\n", file);
#endif
	fordulo->output.file = fp;
	free(file);

	return 1;
    }    
#ifdef DEBUG
    debug(fordulo, "output_open: no open\n");
#endif
    return 0;
}

extern int output_close(t_fordulo * fordulo)
{
	if (fordulo->output.file)
	{
	    fflush(fordulo->output.file);
	    if (fordulo->output.file != stdout)
	    {
		fclose(fordulo->output.file);
	    }
	}
	if (fordulo->output.name)
	{
	    free(fordulo->output.name);
	    fordulo->output.name = NULL;
	}
	if (fordulo->kiir.file)
	{
	    fflush(fordulo->kiir.file);
	    if (fordulo->kiir.file != stdout)
	    {
		fclose(fordulo->kiir.file);
	    }
	}
	if (fordulo->kiir.name)
	{
	    free(fordulo->kiir.name);
	    fordulo->kiir.name = NULL;
	}

    return 1;
}
