/*
 * Copyright (C) 2003  Vizi Gabor (vizi@csucs.net)
 *
 * Ez az adatallomany a reformat program forraskodjanak resze.
 *
 * A program hasznalata, masolasa, terjesztese maganszemely 
 * bevetelt nem eredmenyezo celjara az eredeti forras, 
 * a szerzoi jogi megjegyzes es a hasznalati feltetelek 
 * feltuntetesevel es valtozatlanul hagyasaval, szabadon lehetseges. 
 * A program minden garancia nelkul kerul kozreadasra, a program
 * mukodesebol fakado osszes kockazat a felhasznalot terheli. 
 */
#define PHUA_INTERNAL_ACCESS
#include "include.h"
#include "phua.h"
#include "tit.h"

static t_phua * phua_list_head = NULL;
static t_phua * phua_list_tail = NULL;

extern t_phua * get_phua_head(void)
{
    return phua_list_head;
}

extern t_phua * phua_create(char * feladat, char * megfejtes)
{
    t_phua * temp;
    char * f;
    char * m;
    unsigned long kod = 0;
    unsigned long flen = 0;
    int i;
    
    if (!(temp = malloc(sizeof(t_phua))) || !(f = strdup(feladat)) || !(m = strdup(megfejtes)))
    {
#ifdef DEBUG
	debug(NULL, "phua_create: could not allocate memory for temp\n");
#endif
	return NULL;
    }
    
    memset(temp,0,sizeof(t_phua));
    str_tolower2(f);
    str_tolower2(m);
    
    for (i = 0; i < strlen(f); i++)
    {
	if (islower2(f[i]))
	{
		kod += f[i];
		flen++;
	}
    }

    temp->kod = kod;
    temp->flen = flen;
    temp->jlk = tit_get_jlk(megfejtes);
    temp->feladat = f;
    temp->megfejtes = m;
    temp->next = NULL;
    
    return temp;
}

extern int phua_list_add_phua(char * feladat, char * megfejtes)
{
    t_phua * temp;
    
    if (!(temp = phua_create(feladat, megfejtes)))
    {
	return -1;
    }
    
    if (!phua_list_head)
    {
	phua_list_head = temp;
    }
    if (phua_list_tail)
    {
	phua_list_tail->next = temp;
    }
    phua_list_tail = temp;
#ifdef DEBUG
    debug(NULL, "phua_list_add_phua: %s %s (%d): kod=%d, flen=%d\n", temp->feladat, temp->megfejtes, temp->jlk, temp->kod, temp->flen);
#endif
    return 0;
}

extern int phua_list_destroy(void)
{
    t_phua * temp;
    t_phua * next;
    
    for (temp = phua_list_head; temp; temp = next)
    {
	next = temp->next;
	if (temp->feladat)
	{
	    free(temp->feladat);
	}
	if (temp->megfejtes)
	{
	    free(temp->megfejtes);
	}
	free(temp);
    }
    phua_list_head = NULL;

    return 0;
}


extern int init_phua(char * filename)
{
    FILE * fp;
    char line[LINE_MAXLEN];
    int i;
    int currline;
    char * feladat;
    char * megfejtes;
    
    if (phua_list_head != NULL)
    {
	return 0;
    }
#ifdef DEBUG
    debug(NULL, "init_phua start\n");
#endif
    if(fp = fopen(filename,"rb"))
    {
#ifdef DEBUG
	    debug(NULL, "init_phua open\n");
#endif    
	for (currline = 1; fgets(line, LINE_MAXLEN - 1, fp); currline++)
	{
#ifdef DEBUG
//	    debug(NULL, "init_phua line: %s\n", line);
#endif    
	    if (!line || (*line == '#') || (*line == '\r') || (*line == '\n') || (*line == '\0'))
	    {
		continue;
	    }

	    if (*(line + strlen(line) - 1) == '\n')
	    {
		*(line + strlen(line) - 1) = '\0';
	    }
	    if (*(line + strlen(line) - 1) == '\r')
	    {
		*(line + strlen(line) - 1) = '\0';
	    }

	    feladat = strtok(line, "|");
	    megfejtes = strtok(NULL, "(");
	    feladat = skip_space(feladat);
	    megfejtes = skip_space(megfejtes);
	    
	    if (!strlen(feladat) || !strlen(megfejtes))
	    {
		kiir(NULL, "Hiba a %s conffile %d. soraban (nev)\r\n", filename, currline, i);
		continue;
	    }
	    
	    phua_list_add_phua(feladat, megfejtes);
	    
	}
	fclose(fp);
#ifdef DEBUG
    debug(NULL, "init_phua done\n");
#endif
	return 1;
    }
#ifdef DEBUG
    debug(NULL, "init_phua none\n");
#endif
    return 0;
}

extern char * phua_megfejt(char * feladat, int * jlk)
{
    char * ret = NULL;
    int i;
    int j;
    unsigned long kod = 0;
    unsigned long flen = 0;
    t_phua * temp;
    
    *jlk = 0;
    str_tolower2(feladat);
    
    for (i = 0; i < strlen(feladat); i++)
    {
	if (feladat[i] == '<')
	{
	    for ( ; feladat[i] != '>'; i++);
	    continue;
	}
	if (islower2(feladat[i]))
	{
	    kod += feladat[i];
	    flen++;
	}
    }
    
    if (!phua_list_head)
    {
	load_txtfiles("file_phua");
    }
#ifdef DEBUG
    debug(NULL, "phua_megfejt: %s kod=%d flen=%d\n", feladat, kod, flen);
#endif
    for (temp = phua_list_head; temp; temp = temp->next)
    {
#ifdef DEBUG
    debug(NULL, "phua_megfejt: %s kod=%d flen=%d\n", temp->feladat, temp->kod, temp->flen);
#endif
	if ((temp->kod == kod) && (temp->flen == flen))
	{
	    *jlk = temp->jlk;
	    ret = temp->megfejtes;
	    break;
	}
    }    
#ifdef DEBUG
    debug(NULL, "phua_megfejt: jlk=%d, megfejtes=\"%s\"\n", *jlk, ret);
#endif
    return ret;
}
