/*
 * Copyright (C) 2003  Vizi Gabor (vizi@csucs.net)
 *
 * Ez az adatallomany a reformat program forraskodjanak resze.
 *
 * A program hasznalata, masolasa, terjesztese maganszemely 
 * bevetelt nem eredmenyezo celjara az eredeti forras, 
 * a szerzoi jogi megjegyzes es a hasznalati feltetelek 
 * feltuntetesevel es valtozatlanul hagyasaval, szabadon lehetseges. 
 * A program minden garancia nelkul kerul kozreadasra, a program
 * mukodesebol fakado osszes kockazat a felhasznalot terheli. 
 */
#define SPELL_INTERNAL_ACCESS
#include "include.h"
#include "spell.h"
#include "fordulo.h"
#include "karakter.h"
#include "conf.h"
#include "cache.h"

static int pszi_csere_figyeles = 0;

static t_spell * spell_list_head = NULL;
static t_spell * spell_list_tail = NULL;
static t_spell_pointers * spell_pointers = NULL;
static int spell_ssz_max = 0;

static t_spell_figyelt * spell_figyelt_list_head = NULL;
static t_spell_figyelt * spell_figyelt_list_tail = NULL;

static char * spelltipus_neve[] =
{
    "Teolgia",
    "Taumaturgia",
    "steolgia",
    "staumaturgia",
    "Psz",
    "Alakvlt kpessg",
    NULL
};



extern int spell_get_max(void)
{
    return spell_ssz_max;
}

extern t_spell_figyelt * get_spell_figyelt_head(void)
{
    return spell_figyelt_list_head;
}

t_spell_figyelt * spell_figyelt_create(int ssz, t_confset confset, int irany)
{
    t_spell_figyelt * temp;

    if (!(temp = malloc(sizeof(t_spell_figyelt))))
    {
#ifdef DEBUG
	debug(NULL, "spell_figyel_create: could not allocate memory for temp\n");
#endif
	return NULL;
    }
    
    memset(temp,0,sizeof(t_spell_figyelt));
    temp->ssz = ssz;
    temp->confset = confset;
    temp->irany = irany;
    temp->fentvan = 0;
    
    return temp;
}

extern int spell_figyelt_init(int ssz)
{
    t_spell * spell;
    t_spell_figyelt * temp;
    int ret = 0;
    
    for (temp = spell_figyelt_list_head; temp; temp = temp->next)
    {
	if (!temp->spell && (spell = get_spell_from_ssz(temp->ssz)))
	{
	    temp->spell = spell;
	}
	if (temp->spelllist)
	{
	    temp->spelllist = NULL;
	}
    }

    return ret;
}

extern int spell_figyelt_set_spelllist(int ssz, t_karakter_spelllist * karakter_spelllist)
{
    t_spell_figyelt * temp;
    int ret = 0;
    
    for (temp = spell_figyelt_list_head; temp; temp = temp->next)
    {
	if (temp->ssz == ssz)
	{
	    temp->spelllist = karakter_spelllist;
	    ret = ssz;
	    break;
	}
    }

    return ret;
}

extern int spell_figyelt_set_fentvan(int ssz)
{
    t_spell_figyelt * temp;
    int ret = 0;
    
    for (temp = spell_figyelt_list_head; temp; temp = temp->next)
    {
	if (temp->ssz == ssz)
	{
	    temp->fentvan = 1;
	    ret = ssz;
	    break;
	}
    }

    return ret;
}

extern int spell_figyelt_check(int ssz)
{
    t_spell_figyelt * temp;
    int ret = 0;
    
    for (temp = spell_figyelt_list_head; temp; temp = temp->next)
    {
	if (temp->ssz == ssz)
	{
	    ret = ssz;
	    break;
	}
    }

    return ret;
}

extern int spell_figyelt_add(int ssz, t_confset confset, int irany)
{
    t_spell_figyelt * temp;
    t_spell_figyelt * curr;
    t_spell_figyelt * prev;

    if (!(temp = spell_figyelt_create(ssz, confset, irany)))
    {
	return -1;
    }
    
    for (prev = NULL, curr = spell_figyelt_list_head; curr; prev = curr, curr = curr->next)
    {
	if (curr->ssz == ssz)
	{
	    break;
	}
    }
    
    if (prev)
    {
	temp->next = curr;
	prev->next = temp;
    }
    else
    {
	if (!spell_figyelt_list_head)
	{
	    spell_figyelt_list_head = temp;
	}
	if (spell_figyelt_list_tail)
	{
	    spell_figyelt_list_tail->next = temp;
	}
	spell_figyelt_list_tail = temp;
    }
#ifdef DEBUG
    debug(NULL, "add_spell_figyelt: #%d confset=%d\n", temp->ssz, temp->confset);
#endif
    return 1;
}

extern int spell_figyelt_del_confset(t_confset confset)
{
    t_spell_figyelt * curr;
    t_spell_figyelt * prev;
    t_spell_figyelt * next;
    
    for (curr = spell_figyelt_list_head, prev = NULL; curr; curr = next)
    {
	next = curr->next;
	
	if (curr->confset == confset)
	{
	    if (curr == spell_figyelt_list_head)
	    {
	        spell_figyelt_list_head = curr->next;
	    }
	    if (prev)
	    {
	        prev->next = curr->next;
	    }
	    free(curr);
	    continue;
	}
	prev = curr;
    }
    spell_figyelt_list_tail = prev;
    
    return 0;
}

extern int spell_figyelt_destroy(void)
{
    t_spell_figyelt * temp;
    t_spell_figyelt * next;
    
    for (temp = spell_figyelt_list_head; temp; temp = next)
    {
	next = temp->next;
	free(temp);
    }

    spell_figyelt_list_head = NULL;
    spell_figyelt_list_tail = NULL;

    return 0;
}

extern char * get_spelltipus_neve(int i)
{
    if (i < (sizeof(spelltipus_neve)/sizeof(char*)))
    {
	return spelltipus_neve[i];
    }
    return NULL;
}

extern t_spell * get_spell_from_ssz(int ssz)
{
    if ((spell_ssz_max >= ssz) && spell_pointers && spell_pointers[ssz].spell)
    {
	return spell_pointers[ssz].spell;
    }
    return NULL;
}

extern t_spell * get_spell_head(void)
{
    return spell_list_head;
}

extern t_spell * spell_create(int ssz, char * nev, t_spell_type type, int maxero, int x, int y, int z, t_spell_mento mento)
{
    t_spell * temp;
    char * s;
    
    if (!(temp = malloc(sizeof(t_spell))))
    {
#ifdef DEBUG
	debug(NULL, "spell_create: could not allocate memory for temp\n");
#endif
	return NULL;
    }
    
    if (!(s = strdup(nev)))
    {
	s = NULL;
    }

    memset(temp,0,sizeof(t_spell));

    temp->ssz = ssz;
    temp->nev = s;
    temp->maxero = maxero;
    temp->sebzes.darab = x;
    temp->sebzes.ertek = y;
    temp->sebzes.bonusz = z;
    temp->type = type;
    temp->mento = mento;
    temp->next = NULL;
    
    return temp;
}

extern int spell_list_add_spell(int ssz, char * nev, t_spell_type type, int maxero, int x, int y, int z, t_spell_mento mento)
{
    t_spell * temp = NULL;
    
    if (!(temp = spell_create(ssz, nev, type, maxero, x, y, z, mento)))
    {
	return -1;
    }
    
    if (!spell_list_head)
    {
	spell_list_head = temp;
    }
    if (spell_list_tail)
    {
	spell_list_tail->next = temp;
    }
    spell_list_tail = temp;
    
#ifdef DEBUG
    debug(NULL, "spell_list_add_spell: %s (#%d) mento=%d\n", temp->nev, temp->ssz, temp->mento);
#endif
    return 1;
}

extern int spell_list_destroy(void)
{
    t_spell * temp;
    t_spell * next;
    
    for (temp = spell_list_head; temp; temp = next)
    {
	next = temp->next;
	if (temp->nev)
	{
	    free(temp->nev);
	}
	if (temp->backup)
	{
	    free(temp->backup);
	}
	free(temp);
    }
    spell_list_head = NULL;
    spell_list_tail = NULL;

    free(spell_pointers);
    spell_pointers = NULL;
    return 0;
}

extern t_spell_type spell_get_type_from_char(char c)
{
    t_spell_type type = spell_type_none;
    
    if (c == 'V')
    {
	type |= spell_type_villam;
    }
    else if (c == 'T')
    {
	type |= spell_type_tuz;
    }
    else if (c == 'J')
    {
	type |= spell_type_jeg;
    }
    else if (c == 'M')
    {
	type |= spell_type_mereg;
    }
    else if (c == 'H')
    {
	type |= spell_type_halal;
    }
    else if (c == 'N')
    {
	type |= spell_type_nekrocsok;
    }
    return type;
}

extern spellek_spelllista_check_init(t_fordulo * fordulo)
{
    struct tm * tmbetap;
    t_spell * spell;
    t_spell * temp;
    
    if (!fordulo)
    {
	return -1;
    }
    
    if (pszi_csere_figyeles == 1)
    {
	for (spell = spell_list_head; spell; spell = spell->next)
	{
	    if (spell->backup)
	    {
		spell->maxero = spell->backup->maxero;
		spell->sebzes.darab = spell->backup->sebzes.darab;
		spell->sebzes.ertek = spell->backup->sebzes.ertek;
		spell->sebzes.bonusz = spell->backup->sebzes.bonusz;
		free(spell->backup);
		spell->backup = NULL;
	    }
	}
	pszi_csere_figyeles = 0;
    }
        
    if (fordulo->betap && (fordulo->betap < PSZI_VALTOZAS_TIME))
    {
	if (spell = get_spell_from_ssz(197))
	{
	    if (temp = malloc(sizeof(t_spell)))
	    {
		memset(temp, 0, sizeof(t_spell));
		temp->maxero = spell->maxero;
		temp->sebzes.darab = spell->sebzes.darab;
		temp->sebzes.ertek = spell->sebzes.ertek;
		temp->sebzes.bonusz = spell->sebzes.bonusz;
		
		if (spell->backup)
		{
		    free(spell->backup);
		}
		spell->backup = temp;
		spell->maxero = 99;
		spell->sebzes.darab = 1;
		spell->sebzes.ertek = 6;
		spell->sebzes.bonusz = 0;
		pszi_csere_figyeles = 1;
	    }
	}
	if (spell = get_spell_from_ssz(248))
	{
	    if (temp = malloc(sizeof(t_spell)))
	    {
		memset(temp, 0, sizeof(t_spell));
		temp->maxero = spell->maxero;
		temp->sebzes.darab = spell->sebzes.darab;
		temp->sebzes.ertek = spell->sebzes.ertek;
		temp->sebzes.bonusz = spell->sebzes.bonusz;
		
		if (spell->backup)
		{
		    free(spell->backup);
		}
		spell->backup = temp;
		spell->maxero = 50;
		spell->sebzes.darab = 1;
		spell->sebzes.ertek = 10;
		spell->sebzes.bonusz = 0;
		pszi_csere_figyeles = 1;
	    }
	}
    }
    
    karakter_spelllist_init(fordulo->karakter);
    
    return 0;
}

extern int spellek_spelllista_check(t_fordulo * fordulo)
{
    char * buf = NULL;
    char * buf_vege = NULL;
    char tmp;
    char * p1;
    char * p2;
    char * p3;
    int ssz;
    char szn[32];
    t_spell * spell;
    int i;
    char tol[5];
    char ig[5];
    
    if (!fordulo || !fordulo->karakter || !fordulo->karakter->spellek)
    {
	return -1;
    }

    buf = fordulo->karakter->spellek;
    buf_vege = strstr(buf, "</TABLE>");
    if (!buf || !buf_vege)
    {
	return -1;
    }
    
    if (spellek_spelllista_check_init(fordulo) < 0)
    {
	return -1;
    }
    
    tmp = *buf_vege;
    *buf_vege = '\0';

    p1 = buf;

    if ((fordulo->tipus & fordulo_tipus_reformat_kws) && strstr_(p1, "</a>", buf + strlen(buf)))
    {
	sprintf(tol, ")\">");
	sprintf(ig, "<");
    }
    else
    {
	sprintf(tol, "(#");
	sprintf(ig, ")");
    }
    
    for (p3 = p1, i = 1; (p1 = strstr2(p3, tol)) && (p2 = strstr(p1, ig)); p3 = p2, i++)
    {
	ssz = atoi(p1);
	if ((spell_ssz_max >= ssz) && spell_pointers && spell_pointers[ssz].spell)
	{
	    spell = spell_pointers[ssz].spell;
	    karakter_spelllist_add(fordulo->karakter, spell);
	    {
		int cbuf[3];
		
		cbuf[0] = ssz;
		cbuf[1] = fordulo->fsz;
		cbuf[2] = (int)cache_spell_flag_ismert;
		cache_add_spell(fordulo->cache, cbuf, sizeof(cbuf)/sizeof(int));
	    }
	}
	else
	{
	    
	}
    }
    *buf_vege = tmp;
}


int init_spell(char * filename)
{
    FILE * fp;
    char line[LINE_MAXLEN];
    int i;
    int currline;
    char * p1;
    char * p2;
    char * nev;
    int ssz;
    int t[4];
    t_spell_type type = spell_type_none;
    t_spell_mento mento = spell_mento_none;
    t_spell * spell;
    int ret = 0;
    
    spell_ssz_max = 0;
#ifdef DEBUG
    debug(NULL, "init_spell start\n");
#endif
    if(fp = fopen(filename,"rb"))
    {
#ifdef DEBUG
	    debug(NULL, "init_spell open\n");
#endif    
	for (currline = 1; fgets(line, LINE_MAXLEN - 1, fp); currline++)
	{
	    memset(t, 0, sizeof(t));
	    mento = spell_mento_none;
	    type = spell_type_none;
#ifdef DEBUG
//	    debug(NULL, "init_spell line: %s\n", line);
#endif    
	    if (!line || (*line == '#') || (*line == '\r') || (*line == '\n') || (*line == '\0'))
	    {
		continue;
	    }
	    p1 = skip_space(line);
	    if (*p1 == '\0')
	    {
		continue;
	    }

#ifdef DEBUG
//	    debug(NULL, "init_spell open1\n");
#endif    
	    p2 = skip_nonspace(p1);
	    if ((p1 == p2) || !isdigit2(*p1))
	    {
		kiir(NULL, "Hiba a %s conffile %d. soraban (sorszam)\r\n", filename, currline, i);
		continue;
	    }

	    ssz = atoi(p1);
	    
	    if ((p1 = strstr2(p1, "\"")) && (p2 = strstr(p1, "\"")))
	    {
		if (nev = malloc(p2 - p1 + 1))
		{
		    sprintf(nev, "%.*s", p2 - p1, p1);
		}
		else
		{
		    nev = NULL;
		}
	    }
	    else
	    {
		kiir(NULL, "Hiba a %s conffile %d. soraban (nev)\r\n", filename, currline, i);
		continue;
	    }
	    
	    p1 = skip_space(p2+1);
	    p2 = skip_nonspace(p1);
	    if (p1 == p2)
	    {
		kiir(NULL, "Hiba a %s conffile %d. soraban (fajta)\r\n", filename, currline);
		continue;
	    }
#ifdef DEBUG
//	    debug(NULL, "init_spell open3 p1=\"%.*s\"\n", p2 - p1, p1);
#endif    
    	    type = spell_type_none;
	    
	    if (strncmp(p1, "pszi", 4) == 0)
	    {
		type |= spell_type_pszi;
	    }
	    else if (strncmp(p1, "tau", 3) == 0)
	    {
		type |= spell_type_tau;
	    }
	    else if (strncmp(p1, "teo", 3) == 0)
	    {
		type |= spell_type_teo;
	    }
	    else if (strncmp(p1, "ostau", 5) == 0)
	    {
		type |= spell_type_ostau;
	    }
	    else if (strncmp(p1, "osteo", 5) == 0)
	    {
		type |= spell_type_osteo;
	    }
	    else if (strncmp(p1, "avk", 3) == 0)
	    {
		type |= spell_type_avk;
	    }
	    else if (strncmp(p1, "---", 3) == 0)
	    {
		type = spell_type_none;
	    }
	    else
	    {
		
	    kiir(NULL, "Hiba a %s conffile %d. soraban (tipus)\r\n", filename, currline, i);
		continue;
	    }

	    p1 = p2;
	    
	    for (i = 0; i < 4; i++)
	    {
		p1 = skip_space(p1);
#ifdef DEBUG
	    	debug(NULL, "init_spell open3 p1=\"%.4s\"\n", p1);
#endif    
		if (!isdigit2(*p1))
		{
		    break;
		}
		t[i] = atoi(p1);
		p1 = skip_nonspace(p1);
	    }
#ifdef DEBUG
//	    debug(NULL, "init_spell open2\n");
#endif    
	    if (i == 4)
	    {
		p1 = skip_space(p1);
		p2 = skip_nonspace(p1);
#ifdef DEBUG
	    	debug(NULL, "init_spell open4 p1=\"%.*s\"\n", p2 - p1, p1);
#endif    
		if (p1 != p2)
		{
		    type |= spell_get_type_from_char(*p1);
		}
		p1 = skip_space(p2);
		p2 = skip_nonspace(p1);
#ifdef DEBUG
		debug(NULL, "init_spell open5 p1=\"%.*s\", %d\n", p2 - p1, p1, *p1);
#endif    
		if (p1 != p2)
		{
		    if (*p1 == '0')
		    {
			mento = spell_mento_nullazo;
		    }
		    else if (*p1 == 'f')
		    {
			mento = spell_mento_felezo;
		    }
		    else
		    {
			mento = spell_mento_none;
		    }
		}
	    }
#ifdef DEBUG
//	    debug(NULL, "init_spell open6\n");
#endif
	    if (spell_list_add_spell(ssz, nev, type, t[0], t[1], t[2], t[3], mento) > 0)
	    {
		if (ssz > spell_ssz_max)
		{
		    spell_ssz_max = ssz;
		}
	    }
	    if (nev)
	    {
		free(nev);
	    }
	}
	fclose(fp);
    }

    if (spell_pointers = (t_spell_pointers *)malloc((spell_ssz_max+2) * sizeof(t_spell_pointers)))
    {
	memset(spell_pointers, 0, (spell_ssz_max+2) * sizeof(t_spell_pointers));
	for (spell = spell_list_head; spell; spell = spell->next)
	{
#ifdef DEBUG
	    debug(NULL, "init_spell setting spell_pointers: %d\n", spell->ssz);
#endif    
	    spell_pointers[spell->ssz].ssz = spell->ssz;
	    spell_pointers[spell->ssz].spell = spell;
	}
    }
#ifdef DEBUG
    debug(NULL, "init_spell done\n");
#endif
    if (spell_ssz_max > 0)
    {
	ret = 1;
    }
    return ret;
}
