/*
 * Copyright (C) 2003  Vizi Gabor (vizi@csucs.net)
 *
 * Ez az adatallomany a reformat program forraskodjanak resze.
 *
 * A program hasznalata, masolasa, terjesztese maganszemely 
 * bevetelt nem eredmenyezo celjara az eredeti forras, 
 * a szerzoi jogi megjegyzes es a hasznalati feltetelek 
 * feltuntetesevel es valtozatlanul hagyasaval, szabadon lehetseges. 
 * A program minden garancia nelkul kerul kozreadasra, a program
 * mukodesebol fakado osszes kockazat a felhasznalot terheli. 
 */
#define TARGY_INTERNAL_ACCESS
#include "include.h"
#include "targy.h"
#include "spell.h"
#include "fegyver.h"
#include "fordulo.h"
#include "karakter.h"
#include "sebzes.h"
#include "conf.h"
#include "cache.h"

static t_targy * targy_list_head = NULL;
static t_targy * targy_list_tail = NULL;
static t_targy_pointers * targy_pointers = NULL;
static int targy_ssz_max = 0;

static t_targy_figyelt * targy_figyelt_list_head = NULL;
static t_targy_figyelt * targy_figyelt_list_tail = NULL;

static char * tipusok_neve[] =
{
    "Egyebek",
    "Szrfegyverek",
    "Vgfegyverek",
    "tfegyverek",
    "Lfegyverek",
    "Dobfegyverek",
    "Viziharci fegyverek",
    "Pnclok",
    "Pajzsok",
    "Sisakok",
    "Kesztyk",
    "Csizmk",
    "Gyrk",
    "Kpenyek",
    "Karktk",
    "Flbevalk",
    "Nadrgok",
    "Vallvdk",
    "Vizi pnclok",
    "vek",
    "Nyaklncok",
    "Kulcsok",
    "Lttyk s porok",
    "Lfegyver lvedkek",
    "Tartk",
    "rkkvalsg kszerek",
};

static char * spec_tipusok_neve[] =
{
    "",
    "Esthar drgakvek",
    "Boadhun ftisek",
    "Kyorg relikvik",
    "rnytnde tekercsek",
    "Faji pnclok",
    "Cirkuszi trgyak",
    "jfatty szrmazkok",
    "jfatty esszencik",
    "jfatty trgyak",
    "Moa kszerek",
    "Ryuku trgyak"
};

extern int targy_get_max(void)
{
    return targy_ssz_max;
}

extern t_targy_figyelt * get_targy_figyelt_head(void)
{
    return targy_figyelt_list_head;
}

extern t_targy_figyelt * targy_figyelt_create(int ssz, t_confset confset, int min, int max)
{
    t_targy_figyelt * temp;

    if (!(temp = malloc(sizeof(t_targy_figyelt))))
    {
#ifdef DEBUG
	debug(NULL, "targy_figyel_create: could not allocate memory for temp\n");
#endif
	return NULL;
    }
    
    memset(temp,0,sizeof(t_targy_figyelt));
    temp->ssz = ssz;
    temp->confset = confset;
    temp->min = min;
    temp->max = max;
    
    return temp;
}

extern int targy_figyelt_add(t_karakter * karakter, int ssz, t_confset confset, int min, int max)
{
    t_targy_figyelt * temp;
    t_targy_figyelt * curr;
    t_targy_figyelt * prev;
    int ok;
    int ret;

    if (karakter)
    {
	ret = karakter_targy_figyelt_add(karakter, ssz, min, max);
	return ret;
    }
    
    if (!(temp = targy_figyelt_create(ssz, confset, min, max)))
    {
	return -1;
    }
    
    for (ok = 0, prev = NULL, curr = targy_figyelt_list_head; curr; prev = curr, curr = curr->next)
    {
	if (curr->ssz == ssz)
	{
	    ok = 1;
	    break;
	}
    }
    
    if (ok)
    {
	temp->next = curr->next;
	if (prev)
	{
	    prev->next = temp;
	}
	else
	{
	    targy_figyelt_list_head = temp;
	}
	if (curr == targy_figyelt_list_tail)
	{
	    targy_figyelt_list_tail = temp;
	}
	free(curr);
    }
    else
    {
	if (!targy_figyelt_list_head)
	{
	    targy_figyelt_list_head = temp;
	}
	if (targy_figyelt_list_tail)
	{
	    targy_figyelt_list_tail->next = temp;
	}
	targy_figyelt_list_tail = temp;
    }
#ifdef DEBUG
    debug(NULL, "targy_figyelt_add: #%d confset=%d\n", temp->ssz, temp->confset);
#endif
    return 1;
}

extern int targy_figyelt_del_confset(t_confset confset)
{
    t_targy_figyelt * curr;
    t_targy_figyelt * prev;
    t_targy_figyelt * next;
    
    for (curr = targy_figyelt_list_head, prev = NULL; curr; curr = next)
    {
	next = curr->next;
	
	if (curr->confset == confset)
	{
	    if (curr == targy_figyelt_list_head)
	    {
	        targy_figyelt_list_head = curr->next;
	    }
	    if (prev)
	    {
	        prev->next = curr->next;
	    }
#ifdef DEBUG
	    debug(NULL, "targy_figyelt_del_confset: #%d confset=%d\n", curr->ssz, curr->confset);
#endif
	    free(curr);
	    continue;
	}
	prev = curr;
    }
    targy_figyelt_list_tail = prev;
    
    return 0;
}

extern int targy_figyelt_destroy(void)
{
    t_targy_figyelt * temp;
    t_targy_figyelt * next;
    
    for (temp = targy_figyelt_list_head; temp; temp = next)
    {
	next = temp->next;
	free(temp);
    }

    targy_figyelt_list_head = NULL;
    targy_figyelt_list_tail = NULL;

    return 0;
}

extern char * get_tipusok_neve(int i)
{
    if (i < (sizeof(tipusok_neve)/sizeof(char*)))
    {
	return tipusok_neve[i];
    }
    else if ((i > 1000) && ((i - 1000) < (sizeof(spec_tipusok_neve)/sizeof(char*))))
    {
	return spec_tipusok_neve[i - 1000];
    }
    return NULL;
}

extern t_targy * get_targy_from_ssz(int ssz)
{
    if ((targy_ssz_max >= ssz) && targy_pointers && targy_pointers[ssz].targy)
    {
	return targy_pointers[ssz].targy;
    }
    return NULL;
}

extern int targy_head_init(void)
{
    if (targy_list_head || targy_list_tail)
    {
	targy_list_destroy();
    }
}

extern t_targy * get_targy_head(void)
{
    return targy_list_head;
}

extern int targy_check_faj(t_karakter * karakter, int ssz)
{
    int ret;
    int faj;
    
    ret = 1;
    if ((ssz >= 982) && (ssz <= 1062))
    {
	faj = (ssz - 982) / 9 + 1;
	if (faj != karakter->faj)
	{
	    ret = 0;
	}
    }
    
    return ret;
}

extern t_targy * targy_create(int ssz, char * nev, t_targy_tomb * t)
{
    t_targy * temp;
    char * s;
    t_bonuszok * bonuszok = NULL;
    t_kepessegek * kepessegek = NULL;
    t_fegyver * fegyver = NULL;
    t_spell * spell = NULL;
    t_sebzes * sebzes;
    int * i;
    int * j;
    int ok;
    char * p1;
    char * p2;
    int x = 0;
    int y = 0;
    int z = 0;
    t_fegyver_type ftype;
    t_targy_alkoto * a;
    t_targy_alkoto * a_prev;
    
    if (!(temp = malloc(sizeof(t_targy))))
    {
#ifdef DEBUG
	debug(NULL, "targy_create: could not allocate memory for temp\n");
#endif
	return NULL;
    }
    
    memset(temp,0,sizeof(t_targy));
    
    if (!(s = strdup(nev)))
    {
	s = NULL;
    }

    temp->ssz = ssz;
    temp->nev = s;
    
    if (t)
    {
	temp->tipus =t->tipus;
	temp->minsz = t->minsz;
	temp->suly = t->suly;
	temp->ea = t->ea;
	temp->ve = t->ve;
	temp->nonkf = t->nonkf;
	temp->nonslot = t->nonslot;
	if (t->alkoto)
	{
	    a = NULL;
	    a_prev = NULL;
	    
	    for (p1 = t->alkoto; p1 && isdigit2(*p1) && (p2 = strstr2(p1, "T")) && isdigit2(*p2); )
	    {
		if (!(a = malloc(sizeof(t_targy_alkoto))))
		{
		    temp->alkoto = NULL;
		    break;
		}
		a->ssz = atoi(p2);
		a->darab = atoi(p1);
		a->next = NULL;
		if (!temp->alkoto)
		{
		    temp->alkoto = a;
		}
		else if (a_prev)
		{
		    a_prev->next = a;
		}
		a_prev = a;
		
		p1 = strstr2(p1, " ");
		p1 = skip_space(p1);
	    }
	}
	if (t->eszkoz)
	{
	    a = NULL;
	    a_prev = NULL;
	    
	    for (p1 = t->eszkoz; p1 && isdigit2(*p1) && (p2 = strstr2(p1, "T")) && isdigit2(*p2); )
	    {
		if (!(a = malloc(sizeof(t_targy_alkoto))))
		{
		    temp->eszkoz = NULL;
		    break;
		}
		a->ssz = atoi(p2);
		a->darab = atoi(p1);
		a->next = NULL;
		if (!temp->eszkoz)
		{
		    temp->eszkoz = a;
		}
		else if (a_prev)
		{
		    a_prev->next = a;
		}
		a_prev = a;
		
		p1 = strstr2(p1, " ");
		p1 = skip_space(p1);
	    }
	}
	
	if (ssz == 853)
	{
	    temp->nonkf = 0;
	}
	
	if (bonuszok = malloc(sizeof(t_bonuszok)))
	{
	    memset(bonuszok,0,sizeof(t_bonuszok));
	    for (ok = 0, i = (int *) &t->bonuszok, j = (int *) bonuszok; i < (int *)&t->bonuszok + sizeof(t_bonuszok)/sizeof(int); i++, j++)
	    {
		if (*i)
		{
		    ok = 1;
		    *j = *i;
		}
	    }
	    if (ok)
	    {
	    temp->bonuszok = bonuszok;
	    }
	    else
	    {
		free(bonuszok);
	    }
	}

	if (kepessegek = malloc(sizeof(t_kepessegek)))
	{
	    memset(kepessegek,0,sizeof(t_kepessegek));
	    for (ok = 0, i = (int *) &t->kepessegek, j = (int *) kepessegek; i < (int *)&t->kepessegek + sizeof(t_kepessegek)/sizeof(int); i++, j++)
	    {
		if (*i)
		{
		    ok = 1;
		    *j = *i;
		}
	    }
	    if (ok)
	    {
	    temp->kepessegek = kepessegek;
	    }
	    else
	    {
		free(kepessegek);
	    }
	}

	if ((t->tipus == targy_tipus_szuro) || (t->tipus == targy_tipus_vago) || (t->tipus == targy_tipus_uto) || (t->tipus == targy_tipus_lofegyver_lovedek) || (t->tipus == targy_tipus_dobo) || (t->tipus == targy_tipus_vizi) || ((t->tipus == targy_tipus_pajzs) && t->sebf) || (t->tipus == targy_tipus_kesztyu))
	{
	    sebzes = sebzes_create(t->sebf);
	    if (ftype = get_fegyver_type_from_targy_tipus(temp->ssz, t->tipus, t->balkez))
	    {
		temp->fegyver = fegyver_create(temp, ftype, sebzes, t->fvari, t->fvari_ero, t->fvari_esely);
	    }
	    if (sebzes)
	    {
		free(sebzes);
	    }
	}
	
	if (t->spell)
	{
	    temp->spell = get_spell_from_ssz(t->spell);
	}
    }
    if ((ssz >= 747) && (ssz <= 756))
    {
	temp->tipus2 = targy_tipus_esthar;
    }
    else if ((ssz >= 757) && (ssz <= 766))
    {
	temp->tipus2 = targy_tipus_boadhun;
    }
    else if ((ssz >= 767) && (ssz <= 776))
    {
	temp->tipus2 = targy_tipus_kyorg;
    }
    else if ((ssz >= 777) && (ssz <= 786))
    {
	temp->tipus2 = targy_tipus_arnytunde;
    }
    else if ((ssz >= 982) && (ssz <= 1063))
    {
	temp->tipus2 = targy_tipus_faji_targy;
    }
    else if (((ssz >= 811) && (ssz <= 824)) || ((ssz >= 1233) && (ssz <= 1235)))
    {
	temp->tipus2 = targy_tipus_cirkusz;
    }
    else if ((ssz >= 789) && (ssz <= 793))
    {
	temp->tipus2 = targy_tipus_ejfatty_szarmazek;
    }
    else if (((ssz >= 798) && (ssz <= 802)) || (ssz == 808) || ((ssz >= 1186) && (ssz <= 1189)) || ((ssz >= 1198) && (ssz <= 1202)))
    {
	temp->tipus2 = targy_tipus_ejfatty_esszencia;
    }
    else if (((ssz >= 803) && (ssz <= 807)) || (ssz == 809) || (ssz == 810) || ((ssz >= 794) && (ssz <= 797)) || ((ssz >= 1190) && (ssz <= 1197)) || (ssz == 1216))
    {
	temp->tipus2 = targy_tipus_ejfatty_targy;
    }
    else if ((ssz >= 377) && (ssz <= 386))
    {
	temp->tipus2 = targy_tipus_moa;
    }
    else if (((ssz >= 687) && (ssz <= 696)) || (ssz == 724) || ((ssz >= 1152) && (ssz <= 1159) && (ssz != 1155)) || (ssz == 1162) || (ssz == 1224))
    {
	temp->tipus2 = targy_tipus_ryuku;
    }
    return temp;
}

extern int targy_list_add_targy(int ssz, char * nev, t_targy_tomb * t)
{
    t_targy * temp;

    if (!(temp = targy_create(ssz, nev, t)))
    {
	return -1;
    }
    
    if (!targy_list_head)
    {
	targy_list_head = temp;
    }
    if (targy_list_tail)
    {
	targy_list_tail->next = temp;
    }
    targy_list_tail = temp;
#ifdef DEBUG
    {
	debug(NULL, "targy_list_add_targy: %s (#%d) tipus=%d, nonkf=%d, spell=%d\n", temp->nev, temp->ssz, temp->tipus, temp->nonkf, (temp->spell ? temp->spell->ssz : 0));
    }
#endif
    return 1;
}

extern int targy_list_destroy(void)
{
    t_targy * temp;
    t_targy * next;
    t_targy_alkoto * a_temp;
    t_targy_alkoto * a_next;
    
    for (temp = targy_list_head; temp; temp = next)
    {
	next = temp->next;
	if (temp->nev)
	{
	    free(temp->nev);
	}
	if (temp->bonuszok)
	{
	    free(temp->bonuszok);
	}
	if (temp->kepessegek)
	{
	    free(temp->kepessegek);
	}
	if (temp->fegyver)
	{
	    free(temp->fegyver);
	}
	for (a_temp = temp->alkoto; a_temp; a_temp = a_next)
	{
	    a_next = a_temp->next;
	    free(a_temp);
	}
	for (a_temp = temp->eszkoz; a_temp; a_temp = a_next)
	{
	    a_next = a_temp->next;
	    free(a_temp);
	}
	
	free(temp);
    }

    targy_list_head = NULL;
    targy_list_tail = NULL;

    free(targy_pointers);
    targy_pointers = NULL;
    return 0;
}
/*
extern int targyak_targylista_check_init(t_fordulo * fordulo)
{
}    
*/

extern int targyak_targylista_check(t_fordulo * fordulo)
{
    char * buf = NULL;
    char * buf_vege = NULL;
    char tmp;
    char * p1;
    char * p2;
    char * p3;
    char * p4;
    char * p5;
    char * p6;
    int ssz;
    char szn[32];
    t_targy * targy;
    int i;
    int darab;
    t_targy_flag flag;
    t_targy_hol hol;
    char ** l1;
    char tol[5];
    char ig[5];
    
    if (!fordulo || !fordulo->karakter || !fordulo->karakter->targylista)
    {
	return -1;
    }
    buf = fordulo->karakter->targylista;

    if (fordulo->karakter->bank)
    {
	if (fordulo->karakter->targyment)
	{
#ifdef DEBUG
	    debug(fordulo, "targyak_targylista_check: buf_vege targyment\n");
#endif
    	    buf_vege = fordulo->karakter->targyment;
	}
	else if (fordulo->karakter->targyjott)
	{
#ifdef DEBUG
	    debug(fordulo, "targyak_targylista_check: buf_vege targyjott\n");
#endif
	    buf_vege = fordulo->karakter->targyjott;
	}
	else
	{
#ifdef DEBUG
	    debug(fordulo, "targyak_targylista_check: buf_vege karlapend\n");
#endif
	    buf_vege = fordulo->karakter->karlapend;
	}
    }
    else
    {
#ifdef DEBUG
	debug(fordulo, "targyak_targylista_check: buf_vege megterheltseg\n");
#endif
	buf_vege = fordulo->karakter->megterheltseg;
    }
    if (!buf || !buf_vege)
    {
	return -1;
    }
    
    tmp = *buf_vege;
    *buf_vege = '\0';
    
    if (!(p1 = strstr(buf, "ALIGN=RIGHT")))
    {
	p1 = buf + strlen(buf);
    }
    
    karakter_targylist_init(fordulo->karakter);
    
    if ((fordulo->tipus & fordulo_tipus_reformat_kws) && strstr_(p1, "</a>", buf + strlen(buf)))
    {
	sprintf(tol, ")\">");
	sprintf(ig, "<");
    }
    else
    {
	sprintf(tol, "(");
	sprintf(ig, ")");
    }
    
    for (p3 = p1; (p1 = strstr2(p3, tol)) && (p2 = strstr2(p1, ig)) && ((p2 - p1 < 8) || (strstr_(p1, "a bankban lv trgyak nlkl", p2) == p1) || (strstr(p1, "max.") == p1) || (strstr(p1, "BANKSZF") == p1)); p3 = p2)
    {
	if (!isdigit2(*p1))
	{
	    p1++;
	}
	ssz = atoi(p1);
	flag = targy_flag_none;
	if (strstr_(p3, "*", p1))
	{
	    flag |= targy_flag_kf;
	}
	darab = 0;
	if ((p4 = strstr2(p3, "ALIGN=RIGHT")) && (p5 = strstr2(p4, ">")) && (p6 = strstr(p5, "<")) && (p6 == strstr(p5, "</TD>")) && (p6 < p1))
	{
	    p5 = skip_space(p5);
	    if (*p5 == '$')
	    {
		p5++;
		flag |= targy_flag_hu;
	    }
	    if (isdigit2(*p5))
	    {
		darab = atoi(p5);
	    }
	}
	else if ((p5 = strstr2(p3, "COLSPAN=6>[")) && (p6 = strstr(p5, "]<")) && (p6 == strstr(p2, "]</TD>")))
	{
	    p5 = skip_nonspace(p5);
	    p5 = skip_space(p5);
	    if (*p5 == '$')
	    {
		p5++;
		flag |= targy_flag_hu;
	    }
	    if (isdigit2(*p5))
	    {
		darab = atoi(p5);
	    }
	}
	hol = targy_hol_osszesen;
	if (fordulo->karakter->megterheltseg > p1)
	{
	    hol = targy_hol_nalad;
	}
	else if (fordulo->karakter->targyment && (fordulo->karakter->targyment > p1))
	{
	    hol = targy_hol_bankban;
	}
	else if (fordulo->karakter->targyment && (fordulo->karakter->targyment > p1))
	{
	    hol = targy_hol_bankban;
	}
	if ((targy_ssz_max >= ssz) && targy_pointers && targy_pointers[ssz].targy)
	{
	    targy = targy_pointers[ssz].targy;
	    karakter_targylist_add(fordulo->karakter, targy, darab, flag, hol);
	    {
		int cbuf[3];
		
		cbuf[0] = ssz;
		cbuf[1] = fordulo->fsz;
		if (flag&targy_flag_hu)
		{
		    cbuf[2] = (int)cache_item_flag_hu;
		}
		else
		{
		    cbuf[2] = (int)cache_item_flag_ismert;
		}
		cache_add_item(fordulo->cache, cbuf, sizeof(cbuf)/sizeof(int));
	    }
	}
	else
	{
	    
	}
    }
    *buf_vege = tmp;
    
    if (fordulo->karakter->tulajdonsag && fordulo->karakter->fegyverszaki)
    {
	tmp = *fordulo->karakter->fegyverszaki;
	*fordulo->karakter->fegyverszaki = '\0';
	if ((p1 = strstr2(fordulo->karakter->tulajdonsag, "Pnz:")) && (p2 = strstr2(p1, "</TD><TD")) && ((p2-8 == strstr(p1, "<"))) && (p3 = strstr2(p2, ">")) && (p4 = strstr(p3, "<")))
	{
	    skip_space(p3);
	    if (isdigit2(*p3) && (p5 = (strstr(p3, "arany"))) && (p5 < p4))
	    {
		t_targy * tarany;
		int arany;
		
		arany = atoi(p3);
		fordulo->karakter->arany = arany;
		tarany = get_targy_from_ssz(50);
        	karakter_targylist_add(fordulo->karakter, tarany, arany, targy_flag_none, targy_hol_nalad);
	    }
	}
	*fordulo->karakter->fegyverszaki = tmp;
    }
}

extern int init_targy(char * filename)
{
    FILE * fp;
    char line[LINE_MAXLEN];
    int currline;
    int ssz;
    char * p1;
    char * p2;
    char * p3;
    int i;
    int * pi = NULL;
    char * nev = NULL;
    t_targy * targy;
    t_targy_tipus tipus = targy_tipus_none;
    t_targy_tomb t;
    int ret = 0;
    
    targy_head_init();
    memset(&t,0,sizeof(t_targy_tomb));
#ifdef DEBUG
    debug(NULL, "init_targy start\n");
#endif
    if(fp = fopen(filename,"rb"))
    {
#ifdef DEBUG
//	    debug(NULL, "init_targy open\n");
#endif    
	for (currline = 1; fgets(line, LINE_MAXLEN - 1, fp); currline++)
	{
#ifdef DEBUG
//	    debug(NULL, "init_targy line: %s\n", line);
#endif    
	    if (!line || (*line == '#') || (*line == '\r') || (*line == '\n') || (*line == '\0'))
	    {
		continue;
	    }
	    p1 = skip_space(line);
	    if (*p1 == '\0')
	    {
		continue;
	    }
#ifdef DEBUG
//	    debug(NULL, "init_targy open1\n");
#endif    
	    p2 = skip_nonspace(p1);
	    if ((p1 == p2) || !isdigit2(*p1))
	    {
		kiir(NULL, "Hiba a %s conffile %d. soraban (targyszam)\r\n", filename, currline);
		continue;
	    }
	    
	    if (t.alkoto)
	    {
		free(t.alkoto);
		t.alkoto = NULL;
	    }
	    if (t.eszkoz)
	    {
		free(t.eszkoz);
		t.eszkoz = NULL;
	    }
	    if (t.sebf)
	    {
		free(t.sebf);
		t.sebf = NULL;
	    }
	    
	    memset(&t,0,sizeof(t_targy_tomb));
	    ssz = atoi(p1);

	    if ((p1 = strstr2(p1, "\"")) && (p2 = strstr(p1, "\"")))
	    {
		if (nev = malloc(p2 - p1 + 1))
		{
		    sprintf(nev, "%.*s", p2 - p1, p1);
		}
		else
		{
		    nev = NULL;
		}
	    }
	    else
	    {
		kiir(NULL, "Hiba a %s conffile %d. soraban (nev)\r\n", filename, currline, i);
		continue;
	    }
	    
	    p2 = skip_nonspace(p2);
	    
#ifdef DEBUG
//	    debug(NULL, "init_targy open2\n");
#endif    
	    p1 = skip_space(p2);
	    p2 = skip_nonspace(p1);
	    if (p1 != p2)
	    {
#ifdef DEBUG
//		debug(NULL, "init_targy open3 ssz=%d, nev=\"%s\"\n", ssz, nev);
#endif    
		for ( ; (p3 = strstr2(p1, ";")); p1 = p3)
		{
#ifdef DEBUG
//		    debug(NULL, "init_targy open41 ssz=%d, nev=\"%s\"\n", ssz, nev);
#endif    
		    p1 = skip_space(p1);
		    if ((p2 = strstr2(p1, ":")) && (p2 < p3))
		    {
#ifdef DEBUG
			debug(NULL, "init_targy open42 ssz=%d, nev=\"%s\" p1-p3=\"%.*s\"\n", ssz, nev, p3-p1, p1);
#endif    
			if (strncmp(p1, "alkoto:", p2-p1) == 0)
			{
			    *(p3-1) = '\0';
			    t.alkoto = strdup(p2);
			    *(p3-1) = ';';
			}
			else if (strncmp(p1, "eszkoz:", p2-p1) == 0)
			{
			    *(p3-1) = '\0';
			    t.eszkoz = strdup(p2);
			    *(p3-1) = ';';
			}
			else if (strncmp(p1, "suly:", p2-p1) == 0)
			{
			    t.suly = atoi(p2);
			}
			else if (strncmp(p1, "ea:", p2-p1) == 0)
			{
			    t.ea = atoi(p2);
			}
			else if (strncmp(p1, "ve:", p2-p1) == 0)
			{
			    t.ve = atoi(p2);
			}
			else if (strncmp(p1, "tipus:", p2-p1) == 0)
			{
			    t.tipus = atoi(p2);
			}
			else if (strncmp(p1, "minsz:", p2-p1) == 0)
			{
			    t.minsz = atoi(p2);
			}
			else if (strncmp(p1, "minfv:", p2-p1) == 0)
			{
			    t.minfv = atoi(p2);
			}
			else if (strncmp(p1, "magikus:", p2-p1) == 0)
			{
			    t.magikus = atoi(p2);
			}
			else if (strncmp(p1, "ezust:", p2-p1) == 0)
			{
			    t.ezust = atoi(p2);
			}
			else if (strncmp(p1, "balkez:", p2-p1) == 0)
			{
			    t.balkez = atoi(p2);
			}
			else if (strncmp(p1, "nonkf:", p2-p1) == 0)
			{
			    t.nonkf = atoi(p2);
			}
			else if (strncmp(p1, "nonslot:", p2-p1) == 0)
			{
			    t.nonslot = atoi(p2);
			}
			else if (strncmp(p1, "spell:", p2-p1) == 0)
			{
			    t.spell = atoi(p2);
			}
			else if (strncmp(p1, "fvari:", p2-p1) == 0)
			{
			    t.fvari = atoi(p2);
			}
			else if (strncmp(p1, "fvari_ero:", p2-p1) == 0)
			{
			    t.fvari_ero = atoi(p2);
			}
			else if (strncmp(p1, "fvari_esely:", p2-p1) == 0)
			{
			    t.fvari_esely = atoi(p2);
			}
			else if (strncmp(p1, "sebf:", p2-p1) == 0)
			{
			    *(p3-1) = '\0';
			    t.sebf = strdup(p2);
			    *(p3-1) = ';';
			}
			else if (strncmp(p1, "ero:", p2-p1) == 0)
			{
			    t.bonuszok.ero = atoi(p2);
			}
			else if (strncmp(p1, "iq:", p2-p1) == 0)
			{
			    t.bonuszok.iq = atoi(p2);
			}
			else if (strncmp(p1, "ugy:", p2-p1) == 0)
			{
			    t.bonuszok.ugyesseg = atoi(p2);
			}
			else if (strncmp(p1, "egs:", p2-p1) == 0)
			{
			    t.bonuszok.egeszseg = atoi(p2);
			}
			else if (strncmp(p1, "szer:", p2-p1) == 0)
			{
			    t.bonuszok.szerencse = atoi(p2);
			}
			else if (strncmp(p1, "mxtu:", p2-p1) == 0)
			{
			    t.bonuszok.maxtu = atoi(p2);
			}
			else if (strncmp(p1, "maxep:", p2-p1) == 0)
			{
			    t.bonuszok.maxep = atoi(p2);
			}
			else if (strncmp(p1, "maxvp:", p2-p1) == 0)
			{
			    t.bonuszok.maxvp = atoi(p2);
			}
			else if (strncmp(p1, "maxpp:", p2-p1) == 0)
			{
			    t.bonuszok.maxpp = atoi(p2);
			}
			else if (strncmp(p1, "vpreg:", p2-p1) == 0)
			{
			    t.bonuszok.vpreg = atoi(p2);
			}
			else if (strncmp(p1, "ac:", p2-p1) == 0)
			{
			    t.bonuszok.vedekezes = atoi(p2);
			}
			else if (strncmp(p1, "tam:", p2-p1) == 0)
			{
			    t.bonuszok.tamadas = atoi(p2);
			}
			else if (strncmp(p1, "seb:", p2-p1) == 0)
			{
			    t.bonuszok.sebzes = atoi(p2);
			}
			else if ((strncmp(p1, "kep", 3) == 0) && isdigit2(*(p1+3)))
			{
			    i = atoi(p1+3);
			    i = kepesseg_id_from_hyper_id(i);
			    if ((i >= 1) && (i <= sizeof(t_kepessegek)/sizeof(int)))
			    {
				pi = (int *) &t.kepessegek;
				pi += i - 1;
				*pi = atoi(p2);
			    }
#ifdef DEBUG
//			    debug(NULL, "init_targy open43 ssz=%d, nev=\"%s\" kep=%d, n=%d\n", ssz, nev, i, pi-(int *) &t.kepessegek);
#endif    
			}

#ifdef DEBUG
//			debug(NULL, "init_targy open44 ssz=%d, nev=\"%s\"\n", ssz, nev);
#endif    
		    }
		    else
		    {
			break;
		    }
		} //for key:value;
	    }
#ifdef DEBUG
	    debug(NULL, "init_targy open45 ssz=%d, nev=\"%s\"\n", ssz, nev);
#endif    
	    if (targy_list_add_targy(ssz, nev, &t) > 0)
	    {
		if (ssz > targy_ssz_max)
		{
		    targy_ssz_max = ssz;
		}
	    }
	    if (nev)
	    {
		free(nev);
	    }
	    if (t.alkoto)
	    {
		free(t.alkoto);
		t.alkoto = NULL;
	    }
	    if (t.eszkoz)
	    {
		free(t.eszkoz);
		t.eszkoz = NULL;
	    }
	    if (t.sebf)
	    {
		free(t.sebf);
		t.sebf = NULL;
	    }
	    
	}//for line=...
	fclose(fp);
    }

#ifdef DEBUG
    debug(NULL, "init_targy open5 targy_ssz_max=%d\n", targy_ssz_max);
#endif    

    if (targy_pointers = (t_targy_pointers *)malloc((targy_ssz_max+2) * sizeof(t_targy_pointers)))
    {
	memset(targy_pointers, 0, (targy_ssz_max+2) * sizeof(t_targy_pointers));
	for (targy = targy_list_head; targy; targy = targy->next)
	{
#ifdef DEBUG
//	    debug(NULL, "init_targy open51: %d\n", targy->ssz);
#endif    
	    targy_pointers[targy->ssz].ssz = targy->ssz;
	    targy_pointers[targy->ssz].targy = targy;
	}
    }

#ifdef DEBUG
    debug(NULL, "init_targy done\n");
#endif
    if (targy_ssz_max > 0)
    {
	ret = 1;
    }
    return ret;
}

extern int targy_ad_bonuszt_vizben(t_targy * targy)
{
    int ret = 0;

    if (targy)
    {
	if (targy->ssz == 891)
	{
	    ret = 0;
	}
	else {
	    switch(targy->tipus)
	    {
		case targy_tipus_vizi:
		case targy_tipus_gyuru:
	    	case targy_tipus_karkoto:
		case targy_tipus_fulbevalo:
		case targy_tipus_uszogatya:
		case targy_tipus_ov:
	        case targy_tipus_nyaklanc:
		case targy_tipus_orokkevalosag_ekszer:
		    ret = 1;
		    break;
		default:
		    break;
	    }
	}
    }
    return ret;
}
