/*
 * Copyright (C) 2003  Vizi Gabor (vizi@csucs.net)
 *
 * Ez az adatallomany a reformat program forraskodjanak resze.
 *
 * A program hasznalata, masolasa, terjesztese maganszemely 
 * bevetelt nem eredmenyezo celjara az eredeti forras, 
 * a szerzoi jogi megjegyzes es a hasznalati feltetelek 
 * feltuntetesevel es valtozatlanul hagyasaval, szabadon lehetseges. 
 * A program minden garancia nelkul kerul kozreadasra, a program
 * mukodesebol fakado osszes kockazat a felhasznalot terheli. 
 */
#define TIT_INTERNAL_ACCESS
#include "include.h"
#include "tit.h"

static t_tim * tim_list_head = NULL;
static t_tim * tim_list_tail = NULL;
static char jlk_tab[] = "abcdefghijklmnopqrstuvwxyz.,";

extern char tit_char_get_szam(int i)
{
    char c;
    
    if ((i >= 1) && (i <= strlen(jlk_tab)))
    {
	c = jlk_tab[i-1];
    }
    else
    {
	c = '\0';
    }
    return c;
}

extern int tit_szam_get_char(char c)
{
    char * p;
    int k;
    
    if (c == '')
    {
	c = 'i';
    }
    else if (c == '')
    {
	c = 'o';
    }
    else if (c == '')
    {
	c = '';
    }
    else if (c == '')
    {
	c = 'u';
    }
    else if (c == '')
    {
	c = '';
    }
	
    if (p = strchr(jlk_tab, (int)c))
    {
	k = p - jlk_tab + 1;
	if ((k < 0) || (k > strlen(jlk_tab)))
	{
	    k = 0;
	}
    }
    else
    {
	k = 0;
    }

    return k;
}


extern int tit_get_jlk(char * s)
{
    char * p;
    int jlk = 0;
    int i = 0;
    int k = 0;
    
    if (s[0] == '?')
    {
	i = 1;
    }
    
    for ( ; i < 5; i++)
    {
	if (k = tit_szam_get_char(s[i]))
	{
#ifdef DEBUG
	    debug(NULL, "tit_get_jlk: \"%c\" -> %d\n", s[i], k);
#endif
	    if (k < 10)
	    {
		jlk = 10 * jlk + k;
	    }
	    else if (k < 100)
	    {
		jlk = 100 * jlk + k;
	    }
	    else
	    {
		jlk = 0;
		break;
	    }
	}
	
	if (jlk > 1000)
	{
	    if (jlk > 10000)
	    {
		jlk = jlk / 10;
	    }
	    break;
	}
    }

    return jlk;
}


extern t_tim * get_tim_head(void)
{
    return tim_list_head;
}

extern int tim_list_add_tim(char * szoveg)
{
    t_tim * temp;
    
    if (!(temp = tim_create(szoveg)))
    {
	return -1;
    }
    
    if (!tim_list_head)
    {
	tim_list_head = temp;
    }
    if (tim_list_tail)
    {
	temp->ssz = tim_list_tail->ssz + 1;
	tim_list_tail->next = temp;
    }
    tim_list_tail = temp;
#ifdef DEBUG
    debug(NULL, "tim_list_add_tim: [%d] %s -> jelszo_helye=%d, ct={\"%c\" => %d,%d,%d,%d,%d}\n", temp->ssz, temp->szoveg, temp->jelszo_helye, temp->szoveg[temp->ct[0]], temp->ct[0],temp->ct[1],temp->ct[2],temp->ct[3],temp->ct[4]);
#endif

    return 0;
}

extern int tim_list_destroy(void)
{
    t_tim * temp;
    t_tim * next;
    
    for (temp = tim_list_head; temp; temp = next)
    {
	next = temp->next;
	if (temp->szoveg)
	{
//debug(NULL, "aaa1: %s\n", temp->szoveg);
	    free(temp->szoveg);
//debug(NULL, "aaa2\n");
	}
//debug(NULL, "aaa3\n");
	free(temp);
//debug(NULL, "aaa4\n");
    }
    tim_list_head = NULL;
//debug(NULL, "aaa5\n");

    return 0;
}


extern t_tim * tim_create(char * szoveg)
{
    t_tim * temp;
    char * sz;
    int i;
    int j;
    int n;
    int max;
    int jelszo_helye;
    char c;
    char cc[2];
    char cmax;
    int ct[5];
    char * p;
    
#ifdef DEBUG
//    debug(NULL, "tim_create: szoveg1=\"%s\"\n",szoveg);
#endif
    if (!(temp = malloc(sizeof(t_tim))) || !(sz = malloc(strlen(szoveg) + 1)))
    {
#ifdef DEBUG
	debug(NULL, "tim_create: could not allocate memory for temp\n");
#endif
	return NULL;
    }
    memset(temp,0,sizeof(t_tim));
    memset(temp->ct,-1,5);
    sprintf(sz,"");
    cc[1] = '\0';
    
    for (i = 0, j = 0, n = strlen(szoveg); i < n; i++)
    {
	if ((szoveg[i] == ' ') || (szoveg[i] == '-') || tit_szam_get_char(szoveg[i]))
	{
	    sz[j] = szoveg[i];
	    j++;
	}
    }
    sz[j] = '\0';
    str_tolower2(sz);

#ifdef DEBUG
//	debug(NULL, "tim_create: szoveg2=\"%s\"\n",sz);
#endif
    if (!(p = strstr(sz, "-")))
    {
#ifdef DEBUG
	debug(NULL, "tim_create: nincs benne jelszo \"-\" karakter\n");
#endif
	free(sz);
	free(temp);
	return NULL;
    }
    jelszo_helye = p - sz;
    
    if (!strstr(sz, " "))
    {
	temp->type = tim_type_kelet;
	for (i = 0, max = 0, cmax = 'a'; i < strlen(jlk_tab); i++)
	{
	    cc[0] = jlk_tab[i];
	    for (n = 0, p = sz; p && (p = strstr2(p, cc)); n++);
	    if (n > 5)
	    {
		cmax = cc[0];
		break;
	    }
	    if (n > max)
	    {
		max = n;
		cmax = cc[0];
	    }
	}
	cc[0] = cmax;
    
    }
    else
    {
	temp->type = tim_type_nyugat;
	cc[0] = ' ';
    }
//debug(NULL, "tim_create: cc=\"%s\"\n", cc);
    for (i = 0, p = sz; i < 5; i++)
    {
	if (p && (p = strstr(p, cc)) && (p < (sz + strlen(sz))))
	{
	    temp->ct[i] = p - sz;
//debug(NULL, "tim_create: p=\"%.10s\" temp->ct[%d]=%d\n", p, i, temp->ct[i]);
	    p++;
	}
	else
	{
	    break;
	}
    }

    temp->jelszo_helye = jelszo_helye;
    temp->szoveg = sz;
    temp->next = NULL;
    
    return temp;
}

extern int init_tim(char * filename)
{
    FILE * fp;
    char line[LINE_MAXLEN];
    int i;
    int n = 0;
    int last = 0;
    int currline;
    char * szoveg;
    char * sor;
    char store[6 * LINE_MAXLEN];    
    
    if (tim_list_head != NULL)
    {
	return 0;
    }
#ifdef DEBUG
    debug(NULL, "init_tim start\n");
#endif
    if(fp = fopen(filename,"rb"))
    {
#ifdef DEBUG
	    debug(NULL, "init_tim open\n");
#endif    
	sprintf(store, "");
	for (currline = 1; fgets(line, LINE_MAXLEN - 1, fp); currline++)
	{
#ifdef DEBUG
//	    debug(NULL, "init_tim line: %s\n", line);
#endif    
	    if (!line || (*line == '#') || (*line == '\r') || (*line == '\n') || (*line == '\0'))
	    {
		continue;
	    }
	    
	    if (*(line + strlen(line) - 1) == '\n')
	    {
		*(line + strlen(line) - 1) = '\0';
	    }
	    if (*(line + strlen(line) - 1) == '\r')
	    {
		*(line + strlen(line) - 1) = '\0';
	    }
	    
	    sor = skip_space(line);
	    if (!strlen(sor))
	    {
		continue;
	    }

	    if (isdigit2(*sor))
	    {
		n = atoi(sor);
		if ((n != last+1) && (n != last))
		{
		    kiir(NULL, "Hiba a %s conffile %d. soraban (%d. titkosiras utan %d jon es nem %d)\r\n", filename, currline, last, n, last+1);
		    continue;
		}
		
		last = n;
		
		if (!strlen(store))
		{
		    continue;
		}
		
		if (*(store + strlen(store) - 1) == ' ')
		{
		    *(store + strlen(store) - 1) = '\0';
		}
		tim_list_add_tim(store);
		sprintf(store, "");
		continue;
	    }
	    
	    if (strlen(store) + strlen(sor) >= 6*LINE_MAXLEN)
	    {
		kiir(NULL, "Hiba a %s conffile %d. soraban (tul hosszu az %d. titkosiras)\r\n", filename, currline, last);
		sprintf(store, "");
		continue;
	    }
	    
	    sprintf(store,"%s%s%s", store, sor, (strstr(sor, " ") ? " " : ""));
	}
	if (strlen(store))
	{
	    tim_list_add_tim(store);
	}
	fclose(fp);
#ifdef DEBUG
	debug(NULL, "init_tim done\n");
#endif
	return 1;
    }
#ifdef DEBUG
    debug(NULL, "init_tim none\n");
#endif
    return 0;
}

extern char * tim_megfejt(char * szoveg, int * jlk)
{
    int i;
    int n;
    int k;
    int ok = 0;
    t_tim * temp;
    char tim_tab[35];
    char kod[16];
    char c;
    char * megfejtes = NULL;
    char * p;
    int j1;
    int j2;
    
    if (!tim_list_head)
    {
	load_txtfiles("file_titkosiras");
    }
#ifdef DEBUG
    {
	char * dstr;
	if (dstr = malloc(strlen(szoveg) * 3 + 1))
	{
	    sprintf(dstr,"");
	    for (i = 0; i < strlen(szoveg); i++)
	    {
		sprintf(dstr,"%s%d,", dstr, tit_szam_get_char(szoveg[i]));
	    }
	    debug(NULL, "tim2_megfejt: dstr=\"%s\"\n", dstr);
	    free(dstr);
	}
    }
#endif
    
    *jlk = 0;
    memset(tim_tab,0,35);
    memset(kod,0,16);
    
    for (temp = tim_list_head; temp; temp = temp->next)
    {
	if (temp->type == tim_type_kelet)
	{
	    if (strstr(szoveg, " "))
	    {
		continue;
	    }
	    j1 = temp->jelszo_helye + 4;
	    j2 = temp->jelszo_helye + 4;
	}
	else if (temp->type == tim_type_nyugat)
	{
	    if (!strstr(szoveg, " "))
	    {
		continue;
	    }
	    p = temp->szoveg + temp->jelszo_helye;
	    if ((*p != '-'))
	    {
		continue;
	    }
	    p = skip_nonspace(p);
	    p = skip_space(p);
	    j1 = p - temp->szoveg;
	    
	    p = szoveg + temp->jelszo_helye;
	    p = skip_nonspace(p);
	    p = skip_space(p);
	    j2 = p - szoveg;
	}
	else 
	{
	    continue;
	}
		
#ifdef DEBUG
	debug(NULL, "tim2_megfejt: [%d] szoveg1=\"%s\"\n",temp->ssz,szoveg);
	debug(NULL, "tim2_megfejt: [%d] szoveg2=\"%s\"\n",temp->ssz,temp->szoveg);
#endif
#ifdef DEBUG
	debug(NULL, "tim2_megfejt: [%d] jelszo_helye=%d, j1=%d, j2=%d\n", temp->ssz, temp->jelszo_helye, j1, j2);
	debug(NULL, "tim2_megfejt: [%d] jelszo1=\"%.*s\", jelszo2=\"%.*s\"\n", temp->ssz, j1 - temp->jelszo_helye, temp->szoveg + temp->jelszo_helye, j2 -  temp->jelszo_helye, szoveg + temp->jelszo_helye);
#endif
	if (strlen(temp->szoveg) - j1 == strlen(szoveg) - j2)
	{
	    for (ok = 1, i = 0; (temp->ct[i] != -1) && (i < 5); i++)
	    {
		if ((temp->ct[i] >= j1) || (temp->ct[i] >= j2))
		{
		    n = temp->ct[i] + j1 - j2;
		}
		else
		{
		    n = temp->ct[i];
		}
#ifdef DEBUG
		debug(NULL, "tim2_megfejt: [%d] ct[%d]=%d, ct[%d]=%d\n", temp->ssz, temp->ct[0], szoveg[temp->ct[0]], n, szoveg[n]);
#endif
		if (szoveg[temp->ct[0]] != szoveg[n])
		{
		    ok = 0;
		    break;
		}
	    }
	}
	
	if (ok)
	{
	    break;
	}
    }
    
    sprintf(kod,"");
    
    if (ok)
    {
	if (!(megfejtes = strdup(szoveg)))
	{
	    return 0;
	}
	for (i = 0; i < strlen(megfejtes); i++)
	{
	    if (k = tit_szam_get_char(megfejtes[i]))
	    {
		if (!tim_tab[k])
		{
		    if ((i >= j1) || (i >= j2))
		    {
			n = i + j1 - j2;
		    }
		    else if (i >= temp->jelszo_helye)
		    {
			continue;
		    }
		    else
		    {
			n = i;
		    }
		    tim_tab[k] = temp->szoveg[n];
		}
	    }
	}
	for (i = 0; i < strlen(megfejtes); i++)
	{
	    if (k = tit_szam_get_char(megfejtes[i]))
	    {
		megfejtes[i] = tim_tab[k];
	    }
	    else
	    {
		megfejtes[i] = megfejtes[i];
	    }
	}
	
	if (temp->jelszo_helye && j2)
	{
	    sprintf(kod,"%.*s", ((j2 - temp->jelszo_helye) < 16 ? (j2 - temp->jelszo_helye) : 15), megfejtes + temp->jelszo_helye);
	}
	*jlk = tit_get_jlk(kod);
    }
#ifdef DEBUG
    debug(NULL, "tim2_megfejt: kod=%s, jlk=%d, megfejtes=\"%s\"\n", kod, *jlk, megfejtes);
#endif
    return megfejtes;
}
